"""
Code Block Name: Primatives Images/Videos
"""

# Import dependencies
from pathlib import Path

from encord import EncordUserClient, Project
from encord.objects import ChecklistAttribute, Object, ObjectInstance, Option, RadioAttribute, TextAttribute
from encord.objects.coordinates import SkeletonCoordinate, SkeletonCoordinates
from encord.objects.skeleton_template import SkeletonTemplate

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
# SSH_PATH = get_ssh_key() # replace it with ssh key
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
BUNDLE_SIZE = 100

# Create user client using ssh key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Get project for which labels are to be added
project: Project = user_client.get_project(PROJECT_ID)
assert project is not None, "Project not found — check PROJECT_ID"

# Get ontology structure
ontology_structure = project.ontology_structure
assert ontology_structure is not None, "Ontology structure is missing from the project"

# Get the skeleton annotation object
skeleton_ontology_object: Object = ontology_structure.get_child_by_title(title="Strawberries", type_=Object)
assert skeleton_ontology_object is not None, "Object 'Strawberries' not found in ontology"

# Get skeleton template
assert "Triangle" in project.ontology_structure.skeleton_templates, "Skeleton template 'Triangle' not found"
skeleton_template: SkeletonTemplate = project.ontology_structure.skeleton_templates["Triangle"]
assert skeleton_template is not None, "Skeleton template 'Triangle' is None"

# Get skeleton point IDs
skeleton_ids = [coord.feature_hash for coord in skeleton_template.skeleton.values()]
assert skeleton_ids, "Skeleton template 'Triangle' has no keypoints"

# Get radio attribute for strawberry type
strawberry_type_radio_attribute = ontology_structure.get_child_by_title(type_=RadioAttribute, title="Type?")
assert strawberry_type_radio_attribute is not None, "Radio attribute 'Type?' not found"

# Get radio options
albion_option = strawberry_type_radio_attribute.get_child_by_title(type_=Option, title="Albion")
assert albion_option is not None, "Option 'Albion' not found under 'Type?'"

redcoat_option = strawberry_type_radio_attribute.get_child_by_title(type_=Option, title="Redcoat")
assert redcoat_option is not None, "Option 'Redcoat' not found under 'Type?'"

sweet_kiss_option = strawberry_type_radio_attribute.get_child_by_title(type_=Option, title="Sweet Kiss")
assert sweet_kiss_option is not None, "Option 'Sweet Kiss' not found under 'Type?'"

other_strawberry_option = strawberry_type_radio_attribute.get_child_by_title(
    type_=Option, title="Other strawberry type"
)
assert other_strawberry_option is not None, "Option 'Other strawberry type' not found under 'Type?'"

# Albion Qualities
albion_checklist_attribute = ontology_structure.get_child_by_title(type_=ChecklistAttribute, title="Albion Qualities?")
assert albion_checklist_attribute is not None, "Checklist attribute 'Albion Qualities?' not found"

albion_plump_option = albion_checklist_attribute.get_child_by_title(type_=Option, title="Plump")
assert albion_plump_option is not None, "Option 'Plump' not found under 'Albion Qualities?'"

albion_juicy_option = albion_checklist_attribute.get_child_by_title(type_=Option, title="Juicy")
assert albion_juicy_option is not None, "Option 'Juicy' not found under 'Albion Qualities?'"

albion_large_option = albion_checklist_attribute.get_child_by_title(type_=Option, title="Large")
assert albion_large_option is not None, "Option 'Large' not found under 'Albion Qualities?'"

# Redcoat Qualities
redcoat_checklist_attribute = ontology_structure.get_child_by_title(
    type_=ChecklistAttribute, title="Redcoat Qualities?"
)
assert redcoat_checklist_attribute is not None, "Checklist attribute 'Redcoat Qualities?' not found"

redcoat_plump_option = redcoat_checklist_attribute.get_child_by_title(type_=Option, title="Plump")
assert redcoat_plump_option is not None, "Option 'Plump' not found under 'Redcoat Qualities?'"

redcoat_juicy_option = redcoat_checklist_attribute.get_child_by_title(type_=Option, title="Juicy")
assert redcoat_juicy_option is not None, "Option 'Juicy' not found under 'Redcoat Qualities?'"

redcoat_large_option = redcoat_checklist_attribute.get_child_by_title(type_=Option, title="Large")
assert redcoat_large_option is not None, "Option 'Large' not found under 'Redcoat Qualities?'"

# Sweet Kiss Qualities
sweet_kiss_checklist_attribute = ontology_structure.get_child_by_title(
    type_=ChecklistAttribute, title="Sweet Kiss Qualities?"
)
assert sweet_kiss_checklist_attribute is not None, "Checklist attribute 'Sweet Kiss Qualities?' not found"

sweet_kiss_plump_option = sweet_kiss_checklist_attribute.get_child_by_title(type_=Option, title="Plump")
assert sweet_kiss_plump_option is not None, "Option 'Plump' not found under 'Sweet Kiss Qualities?'"

sweet_kiss_juicy_option = sweet_kiss_checklist_attribute.get_child_by_title(type_=Option, title="Juicy")
assert sweet_kiss_juicy_option is not None, "Option 'Juicy' not found under 'Sweet Kiss Qualities?'"

sweet_kiss_large_option = sweet_kiss_checklist_attribute.get_child_by_title(type_=Option, title="Large")
assert sweet_kiss_large_option is not None, "Option 'Large' not found under 'Sweet Kiss Qualities?'"

# Other strawberry type (text input)
other_strawberry_option_text_attribute = ontology_structure.get_child_by_title(
    type_=TextAttribute, title="Specify strawberry type"
)
assert other_strawberry_option_text_attribute is not None, "Text attribute 'Specify strawberry type' not found"

# Dictionary of labels per data unit and per frame with strawberry type specified, including quality options
video_frame_labels = {
    "cherries-001.jpg": {
        0: {
            "label_ref": "strawberry_001",
            "coordinates": SkeletonCoordinates(
                values=[
                    SkeletonCoordinate(
                        x=0.25, y=0.25, name="point_0", color="#000000", value="point_0", feature_hash=skeleton_ids[0]
                    ),
                    SkeletonCoordinate(
                        x=0.35, y=0.25, name="point_1", color="#000000", value="point_1", feature_hash=skeleton_ids[1]
                    ),
                    SkeletonCoordinate(
                        x=0.25, y=0.35, name="point_2", color="#000000", value="point_2", feature_hash=skeleton_ids[2]
                    ),
                ],
                name="Triangle",
            ),
            "strawberry_type": "Albion",
            "albion_quality_options": "Plump, Juicy",
        }
    },
    "cherries-010.jpg": {
        0: [
            {
                "label_ref": "strawberry_002",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.25,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.35,
                            y=0.25,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.35,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Redcoat",
                "redcoat_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "strawberry_003",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.45,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.55,
                            y=0.45,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.55,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Sweet Kiss",
                "sweet_kiss_quality_options": "Plump",
            },
            {
                "label_ref": "strawberry_004",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.65,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.75,
                            y=0.65,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.75,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Other strawberry type",
                "Specify strawberry type": "Pineberry",
            },
        ],
    },
    "cherries-ig": {
        0: {
            "label_ref": "strawberry_005",
            "coordinates": SkeletonCoordinates(
                values=[
                    SkeletonCoordinate(
                        x=0.25, y=0.25, name="point_0", color="#000000", value="point_0", feature_hash=skeleton_ids[0]
                    ),
                    SkeletonCoordinate(
                        x=0.35, y=0.25, name="point_1", color="#000000", value="point_1", feature_hash=skeleton_ids[1]
                    ),
                    SkeletonCoordinate(
                        x=0.25, y=0.35, name="point_2", color="#000000", value="point_2", feature_hash=skeleton_ids[2]
                    ),
                ],
                name="Triangle",
            ),
            "strawberry_type": "Albion",
            "albion_quality_options": "Plump, Juicy",
        },
        2: [
            {
                "label_ref": "strawberry_006",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.25,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.35,
                            y=0.25,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.35,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Redcoat",
                "redcoat_quality_options": "Large",
            },
            {
                "label_ref": "strawberry_007",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.45,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.55,
                            y=0.45,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.55,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Sweet Kiss",
                "sweet_kiss_quality_options": "Plump",
            },
            {
                "label_ref": "strawberry_008",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.65,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.75,
                            y=0.65,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.75,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Other strawberry type",
                "Specify strawberry type": "Pineberry",
            },
        ],
    },
    "cherries-is": {
        0: {
            "label_ref": "strawberry_009",
            "coordinates": SkeletonCoordinates(
                values=[
                    SkeletonCoordinate(
                        x=0.25, y=0.25, name="point_0", color="#000000", value="point_0", feature_hash=skeleton_ids[0]
                    ),
                    SkeletonCoordinate(
                        x=0.35, y=0.25, name="point_1", color="#000000", value="point_1", feature_hash=skeleton_ids[1]
                    ),
                    SkeletonCoordinate(
                        x=0.25, y=0.35, name="point_2", color="#000000", value="point_2", feature_hash=skeleton_ids[2]
                    ),
                ],
                name="Triangle",
            ),
            "strawberry_type": "Albion",
            "albion_quality_options": "Plump",
        },
        3: [
            {
                "label_ref": "strawberry_010",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.25,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.35,
                            y=0.25,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.35,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Redcoat",
                "redcoat_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "strawberry_011",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.45,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.55,
                            y=0.45,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.55,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Sweet Kiss",
                "sweet_kiss_quality_options": "Plump",
            },
            {
                "label_ref": "strawberry_012",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.65,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.75,
                            y=0.65,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.75,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Other strawberry type",
                "Specify strawberry type": "Pineberry",
            },
        ],
    },
    "cherries-vid-001.mp4": {
        103: [
            {
                "label_ref": "strawberry_013",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.25,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.35,
                            y=0.25,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.35,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Sweet Kiss",
                "sweet_kiss_quality_options": "Plump",
            },
            {
                "label_ref": "strawberry_014",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.45,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.55,
                            y=0.45,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.55,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Albion",
                "albion_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "strawberry_015",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.65,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.75,
                            y=0.65,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.75,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Other strawberry type",
                "Specify strawberry type": "Pineberry",
            },
        ],
        104: [
            {
                "label_ref": "strawberry_016",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.15,
                            y=0.15,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.25,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.15,
                            y=0.15,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Sweet Kiss",
                "sweet_kiss_quality_options": "Plump",
            },
            {
                "label_ref": "strawberry_014",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.35,
                            y=0.35,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.35,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.35,
                            y=0.45,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Albion",
                "albion_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "strawberry_017",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.55,
                            y=0.55,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.55,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.55,
                            y=0.65,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Other strawberry type",
                "Specify strawberry type": "Pineberry",
            },
        ],
        105: [
            {
                "label_ref": "strawberry_016",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.25,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.35,
                            y=0.25,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.25,
                            y=0.35,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Sweet Kiss",
                "sweet_kiss_quality_options": "Plump",
            },
            {
                "label_ref": "strawberry_014",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.45,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.55,
                            y=0.45,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.45,
                            y=0.55,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Albion",
                "albion_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "strawberry_017",
                "coordinates": SkeletonCoordinates(
                    values=[
                        SkeletonCoordinate(
                            x=0.75,
                            y=0.75,
                            name="point_0",
                            color="#000000",
                            value="point_0",
                            feature_hash=skeleton_ids[0],
                        ),
                        SkeletonCoordinate(
                            x=0.65,
                            y=0.75,
                            name="point_1",
                            color="#000000",
                            value="point_1",
                            feature_hash=skeleton_ids[1],
                        ),
                        SkeletonCoordinate(
                            x=0.75,
                            y=0.65,
                            name="point_2",
                            color="#000000",
                            value="point_2",
                            feature_hash=skeleton_ids[2],
                        ),
                    ],
                    name="Triangle",
                ),
                "strawberry_type": "Other strawberry type",
                "Specify strawberry type": "Pineberry",
            },
        ],
    },
}


# Loop through each data unit (image, video, etc.)
for data_unit, frame_coordinates in video_frame_labels.items():
    object_instances_by_label_ref = {}

    # Get the label row for the current data unit
    label_rows = project.list_label_rows_v2(data_title_eq=data_unit)
    assert isinstance(label_rows, list), f"Expected list of label rows for '{data_unit}', got {type(label_rows)}"
    assert label_rows, f"No label row found for data unit: {data_unit}"

    label_row = label_rows[0]
    assert label_row is not None, f"Label row is None for {data_unit}"

    # Initialize label row using bundle
    with project.create_bundle(bundle_size=BUNDLE_SIZE) as bundle:
        label_row.initialise_labels()
        assert label_row.ontology_structure is not None, f"Ontology not initialized for label row: {data_unit}"

        for frame_number, items in frame_coordinates.items():
            assert isinstance(frame_number, int), f"Frame number must be int, got {type(frame_number)}"
            if not isinstance(items, list):
                items = [items]

            for item in items:
                label_ref = item["label_ref"]
                coord = item["coordinates"]
                strawberry_type = item["strawberry_type"]

                assert strawberry_type in {
                    "Albion",
                    "Redcoat",
                    "Sweet Kiss",
                    "Other strawberry type",
                }, f"Unexpected strawberry type '{strawberry_type}' in data unit '{data_unit}'"

                if label_ref not in object_instances_by_label_ref:
                    skeleton_object_instance: ObjectInstance = skeleton_ontology_object.create_instance()
                    assert skeleton_object_instance is not None, f"Failed to create ObjectInstance for {label_ref}"

                    object_instances_by_label_ref[label_ref] = skeleton_object_instance
                    checklist_attribute = None

                    # Set strawberry type radio attribute
                    if strawberry_type == "Albion":
                        assert albion_option is not None, "Missing 'albion_option'"
                        skeleton_object_instance.set_answer(
                            attribute=strawberry_type_radio_attribute, answer=albion_option
                        )
                        checklist_attribute = albion_checklist_attribute
                    elif strawberry_type == "Redcoat":
                        assert redcoat_option is not None, "Missing 'redcoat_option'"
                        skeleton_object_instance.set_answer(
                            attribute=strawberry_type_radio_attribute, answer=redcoat_option
                        )
                        checklist_attribute = redcoat_checklist_attribute
                    elif strawberry_type == "Sweet Kiss":
                        assert sweet_kiss_option is not None, "Missing 'sweet_kiss_option'"
                        skeleton_object_instance.set_answer(
                            attribute=strawberry_type_radio_attribute, answer=sweet_kiss_option
                        )
                        checklist_attribute = sweet_kiss_checklist_attribute
                    elif strawberry_type == "Other strawberry type":
                        assert other_strawberry_option is not None, "Missing 'other_strawberry_option'"
                        skeleton_object_instance.set_answer(
                            attribute=strawberry_type_radio_attribute, answer=other_strawberry_option
                        )
                        text_answer = item.get("Specify strawberry type", "")
                        assert isinstance(text_answer, str), "'Specify strawberry type' must be a string"
                        skeleton_object_instance.set_answer(
                            attribute=other_strawberry_option_text_attribute,
                            answer=text_answer,
                        )

                    # Set checklist attributes
                    checklist_answers = []
                    quality_key = f"{strawberry_type.lower().replace(' ', '_')}_quality_options"
                    quality_options = item.get(quality_key, "").split(", ")

                    for quality in quality_options:
                        option = None
                        if quality == "Plump":
                            option = (
                                albion_plump_option
                                if strawberry_type == "Albion"
                                else redcoat_plump_option
                                if strawberry_type == "Redcoat"
                                else sweet_kiss_plump_option
                                if strawberry_type == "Sweet Kiss"
                                else None
                            )
                        elif quality == "Juicy":
                            option = (
                                albion_juicy_option
                                if strawberry_type == "Albion"
                                else redcoat_juicy_option
                                if strawberry_type == "Redcoat"
                                else sweet_kiss_juicy_option
                                if strawberry_type == "Sweet Kiss"
                                else None
                            )
                        elif quality == "Large":
                            option = (
                                albion_large_option
                                if strawberry_type == "Albion"
                                else redcoat_large_option
                                if strawberry_type == "Redcoat"
                                else sweet_kiss_large_option
                                if strawberry_type == "Sweet Kiss"
                                else None
                            )

                        if option:
                            checklist_answers.append(option)
                        else:
                            assert strawberry_type == "Other strawberry type", (
                                f"Invalid quality '{quality}' for strawberry type '{strawberry_type}'"
                            )

                    if checklist_attribute and checklist_answers:
                        skeleton_object_instance.set_answer(
                            attribute=checklist_attribute, answer=checklist_answers, overwrite=True
                        )

                else:
                    skeleton_object_instance = object_instances_by_label_ref[label_ref]

                # Assign the object to the frame and track it
                skeleton_object_instance.set_for_frames(coordinates=coord, frames=frame_number)

        # Add object instances to label_row **only if they have frames assigned**
        for skeleton_object_instance in object_instances_by_label_ref.values():
            assert isinstance(skeleton_object_instance, ObjectInstance), "Expected ObjectInstance type"
            if skeleton_object_instance.get_annotation_frames():
                label_row.add_object_instance(skeleton_object_instance)

        # Save label row using the bundle
        assert label_row is not None, "Trying to save a None label row"
        label_row.save(bundle=bundle)

print("Labels with strawberry type radio buttons, checklist attributes, and text labels added for all data units.")



"""
Code Block Name: Simple
"""

# Import dependencies
from pathlib import Path

from encord import EncordUserClient, Project
from encord.objects import ChecklistAttribute, Object, ObjectInstance, Option, RadioAttribute, TextAttribute
from encord.objects.coordinates import PointCoordinate, PolygonCoordinates

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
# SSH_PATH = get_ssh_key() # replace it with ssh key
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
BUNDLE_SIZE = 100

# Create user client using ssh key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Get project for which labels are to be added
project: Project = user_client.get_project(PROJECT_ID)
assert project is not None, "Project not found — check PROJECT_ID"

# Get ontology structure
ontology_structure = project.ontology_structure
assert ontology_structure is not None, "Ontology structure is missing in the project"

# Find polygon object for blueberries
polygon_ontology_object: Object = ontology_structure.get_child_by_title(title="Blueberries", type_=Object)
assert polygon_ontology_object is not None, "Object 'Blueberries' not found in ontology"

# Get radio attribute for blueberry type
blueberry_type_radio_attribute = ontology_structure.get_child_by_title(type_=RadioAttribute, title="Type?")
assert blueberry_type_radio_attribute is not None, "Radio attribute 'Type?' not found"

# Get radio options
bluegold_option = blueberry_type_radio_attribute.get_child_by_title(type_=Option, title="Bluegold")
assert bluegold_option is not None, "Option 'Bluegold' not found under radio attribute 'Type?'"

duke_option = blueberry_type_radio_attribute.get_child_by_title(type_=Option, title="Duke")
assert duke_option is not None, "Option 'Duke' not found under radio attribute 'Type?'"

blueray_option = blueberry_type_radio_attribute.get_child_by_title(type_=Option, title="Blueray")
assert blueray_option is not None, "Option 'Blueray' not found under radio attribute 'Type?'"

other_blueberry_option = blueberry_type_radio_attribute.get_child_by_title(type_=Option, title="Other blueberry type")
assert other_blueberry_option is not None, "Option 'Other blueberry type' not found under radio attribute 'Type?'"

# Bluegold Qualities
bluegold_checklist_attribute = ontology_structure.get_child_by_title(
    type_=ChecklistAttribute, title="Bluegold Qualities?"
)
assert bluegold_checklist_attribute is not None, "Checklist attribute 'Bluegold Qualities?' not found"

bluegold_plump_option = bluegold_checklist_attribute.get_child_by_title(type_=Option, title="Plump")
assert bluegold_plump_option is not None, "Option 'Plump' not found under 'Bluegold Qualities?'"

bluegold_juicy_option = bluegold_checklist_attribute.get_child_by_title(type_=Option, title="Juicy")
assert bluegold_juicy_option is not None, "Option 'Juicy' not found under 'Bluegold Qualities?'"

bluegold_large_option = bluegold_checklist_attribute.get_child_by_title(type_=Option, title="Large")
assert bluegold_large_option is not None, "Option 'Large' not found under 'Bluegold Qualities?'"

# Duke Qualities
duke_checklist_attribute = ontology_structure.get_child_by_title(type_=ChecklistAttribute, title="Duke Qualities?")
assert duke_checklist_attribute is not None, "Checklist attribute 'Duke Qualities?' not found"

duke_plump_option = duke_checklist_attribute.get_child_by_title(type_=Option, title="Plump")
assert duke_plump_option is not None, "Option 'Plump' not found under 'Duke Qualities?'"

duke_juicy_option = duke_checklist_attribute.get_child_by_title(type_=Option, title="Juicy")
assert duke_juicy_option is not None, "Option 'Juicy' not found under 'Duke Qualities?'"

duke_large_option = duke_checklist_attribute.get_child_by_title(type_=Option, title="Large")
assert duke_large_option is not None, "Option 'Large' not found under 'Duke Qualities?'"

# Blueray Qualities
blueray_checklist_attribute = ontology_structure.get_child_by_title(
    type_=ChecklistAttribute, title="Blueray Qualities?"
)
assert blueray_checklist_attribute is not None, "Checklist attribute 'Blueray Qualities?' not found"

blueray_plump_option = blueray_checklist_attribute.get_child_by_title(type_=Option, title="Plump")
assert blueray_plump_option is not None, "Option 'Plump' not found under 'Blueray Qualities?'"

blueray_juicy_option = blueray_checklist_attribute.get_child_by_title(type_=Option, title="Juicy")
assert blueray_juicy_option is not None, "Option 'Juicy' not found under 'Blueray Qualities?'"

blueray_large_option = blueray_checklist_attribute.get_child_by_title(type_=Option, title="Large")
assert blueray_large_option is not None, "Option 'Large' not found under 'Blueray Qualities?'"

# Other blueberry type — text input
other_blueberry_option_text_attribute = ontology_structure.get_child_by_title(
    type_=TextAttribute, title="Specify blueberry type"
)
assert other_blueberry_option_text_attribute is not None, "Text attribute 'Specify blueberry type' not found"


# Dictionary of labels per data unit and per frame with blueberry type specified, including quality options
video_frame_labels = {
    "cherries-001.jpg": {
        0: {
            "label_ref": "blueberry_001",
            "coordinates": PolygonCoordinates(
                polygons=[
                    [
                        [
                            PointCoordinate(0.01, 0.02),
                            PointCoordinate(0.03, 0.03),
                            PointCoordinate(0.05, 0.02),
                            PointCoordinate(0.04, 0.01),
                            PointCoordinate(0.02, 0.01),
                        ]
                    ]
                ]
            ),
            "blueberry_type": "Bluegold",
            "bluegold_quality_options": "Plump, Juicy",
        }
    },
    "cherries-010.jpg": {
        0: [
            {
                "label_ref": "blueberry_002",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.01, 0.02),
                                PointCoordinate(0.03, 0.03),
                                PointCoordinate(0.05, 0.02),
                                PointCoordinate(0.04, 0.01),
                                PointCoordinate(0.02, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Duke",
                "duke_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "blueberry_003",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.04, 0.05),
                                PointCoordinate(0.06, 0.06),
                                PointCoordinate(0.08, 0.05),
                                PointCoordinate(0.07, 0.04),
                                PointCoordinate(0.05, 0.04),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Blueray",
                "blueray_quality_options": "Plump",
            },
            {
                "label_ref": "blueberry_004",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.07, 0.02),
                                PointCoordinate(0.09, 0.03),
                                PointCoordinate(0.11, 0.02),
                                PointCoordinate(0.10, 0.01),
                                PointCoordinate(0.08, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Other blueberry type",
                "Specify blueberry type": "Highbush",
            },
        ],
    },
    "cherries-ig": {
        0: {
            "label_ref": "blueberry_005",
            "coordinates": PolygonCoordinates(
                polygons=[
                    [
                        [
                            PointCoordinate(0.01, 0.02),
                            PointCoordinate(0.03, 0.03),
                            PointCoordinate(0.05, 0.02),
                            PointCoordinate(0.04, 0.01),
                            PointCoordinate(0.02, 0.01),
                        ]
                    ]
                ]
            ),
            "blueberry_type": "Bluegold",
            "bluegold_quality_options": "Plump, Juicy",
        },
        2: [
            {
                "label_ref": "blueberry_006",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.01, 0.02),
                                PointCoordinate(0.03, 0.03),
                                PointCoordinate(0.05, 0.02),
                                PointCoordinate(0.04, 0.01),
                                PointCoordinate(0.02, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Duke",
                "duke_quality_options": "Large",
            },
            {
                "label_ref": "blueberry_007",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.04, 0.05),
                                PointCoordinate(0.06, 0.06),
                                PointCoordinate(0.08, 0.05),
                                PointCoordinate(0.07, 0.04),
                                PointCoordinate(0.05, 0.04),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Blueray",
                "blueray_quality_options": "Plump",
            },
            {
                "label_ref": "blueberry_008",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.07, 0.02),
                                PointCoordinate(0.09, 0.03),
                                PointCoordinate(0.11, 0.02),
                                PointCoordinate(0.10, 0.01),
                                PointCoordinate(0.08, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Other blueberry type",
                "Specify blueberry type": "Lowbush",
            },
        ],
    },
    "cherries-is": {
        0: {
            "label_ref": "blueberry_009",
            "coordinates": PolygonCoordinates(
                polygons=[
                    [
                        [
                            PointCoordinate(0.01, 0.02),
                            PointCoordinate(0.03, 0.03),
                            PointCoordinate(0.05, 0.02),
                            PointCoordinate(0.04, 0.01),
                            PointCoordinate(0.02, 0.01),
                        ]
                    ]
                ]
            ),
            "blueberry_type": "Bluegold",
            "bluegold_quality_options": "Plump",
        },
        3: [
            {
                "label_ref": "blueberry_010",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.01, 0.02),
                                PointCoordinate(0.03, 0.03),
                                PointCoordinate(0.05, 0.02),
                                PointCoordinate(0.04, 0.01),
                                PointCoordinate(0.02, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Duke",
                "duke_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "blueberry_011",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.04, 0.05),
                                PointCoordinate(0.06, 0.06),
                                PointCoordinate(0.08, 0.05),
                                PointCoordinate(0.07, 0.04),
                                PointCoordinate(0.05, 0.04),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Blueray",
                "blueray_quality_options": "Plump",
            },
            {
                "label_ref": "blueberry_012",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.07, 0.02),
                                PointCoordinate(0.09, 0.03),
                                PointCoordinate(0.11, 0.02),
                                PointCoordinate(0.10, 0.01),
                                PointCoordinate(0.08, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Other blueberry type",
                "Specify blueberry type": "Highbush",
            },
        ],
    },
    "cherries-vid-001.mp4": {
        103: [
            {
                "label_ref": "blueberry_013",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.01, 0.02),
                                PointCoordinate(0.03, 0.03),
                                PointCoordinate(0.05, 0.02),
                                PointCoordinate(0.04, 0.01),
                                PointCoordinate(0.02, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Blueray",
                "blueray_quality_options": "Plump",
            },
            {
                "label_ref": "blueberry_014",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.04, 0.05),
                                PointCoordinate(0.06, 0.06),
                                PointCoordinate(0.08, 0.05),
                                PointCoordinate(0.07, 0.04),
                                PointCoordinate(0.05, 0.04),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Bluegold",
                "bluegold_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "blueberry_015",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.07, 0.02),
                                PointCoordinate(0.09, 0.03),
                                PointCoordinate(0.11, 0.02),
                                PointCoordinate(0.10, 0.01),
                                PointCoordinate(0.08, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Other blueberry type",
                "Specify blueberry type": "Lowbush",
            },
        ],
        104: [
            {
                "label_ref": "blueberry_016",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.01, 0.02),
                                PointCoordinate(0.03, 0.03),
                                PointCoordinate(0.05, 0.02),
                                PointCoordinate(0.04, 0.01),
                                PointCoordinate(0.02, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Blueray",
                "blueray_quality_options": "Plump",
            },
            {
                "label_ref": "blueberry_014",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.041, 0.052),
                                PointCoordinate(0.061, 0.062),
                                PointCoordinate(0.081, 0.052),
                                PointCoordinate(0.071, 0.042),
                                PointCoordinate(0.051, 0.042),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Bluegold",
                "bluegold_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "blueberry_017",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.07, 0.02),
                                PointCoordinate(0.09, 0.03),
                                PointCoordinate(0.11, 0.02),
                                PointCoordinate(0.10, 0.01),
                                PointCoordinate(0.08, 0.01),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Other blueberry type",
                "Specify blueberry type": "Highbush",
            },
        ],
        105: [
            {
                "label_ref": "blueberry_016",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.011, 0.022),
                                PointCoordinate(0.031, 0.032),
                                PointCoordinate(0.051, 0.022),
                                PointCoordinate(0.041, 0.012),
                                PointCoordinate(0.021, 0.012),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Blueray",
                "blueray_quality_options": "Plump",
            },
            {
                "label_ref": "blueberry_014",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.043, 0.055),
                                PointCoordinate(0.063, 0.065),
                                PointCoordinate(0.083, 0.055),
                                PointCoordinate(0.073, 0.045),
                                PointCoordinate(0.053, 0.045),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Bluegold",
                "bluegold_quality_options": "Plump, Juicy, Large",
            },
            {
                "label_ref": "blueberry_017",
                "coordinates": PolygonCoordinates(
                    polygons=[
                        [
                            [
                                PointCoordinate(0.071, 0.022),
                                PointCoordinate(0.091, 0.032),
                                PointCoordinate(0.111, 0.022),
                                PointCoordinate(0.101, 0.012),
                                PointCoordinate(0.081, 0.012),
                            ]
                        ]
                    ]
                ),
                "blueberry_type": "Other blueberry type",
                "Specify blueberry type": "Highbush",
            },
        ],
    },
}

# Loop through each data unit (image, video, etc.)
for data_unit, frame_coordinates in video_frame_labels.items():
    object_instances_by_label_ref = {}

    # Get the label row for the current data unit
    label_rows = project.list_label_rows_v2(data_title_eq=data_unit)
    assert isinstance(label_rows, list), f"Expected list of label rows for '{data_unit}', got {type(label_rows)}"
    assert label_rows, f"No label rows found for data unit: {data_unit}"

    label_row = label_rows[0]
    assert label_row is not None, f"Label row is None for {data_unit}"

    # Initialize label row using bundle
    with project.create_bundle(bundle_size=BUNDLE_SIZE) as bundle:
        label_row.initialise_labels()
        assert label_row.ontology_structure is not None, f"Ontology not initialized for label row: {data_unit}"

        for frame_number, items in frame_coordinates.items():
            assert isinstance(frame_number, int), f"Frame number must be int, got {type(frame_number)}"

            if not isinstance(items, list):  # Single or multiple objects in the frame
                items = [items]

            for item in items:
                label_ref = item["label_ref"]
                coord = item["coordinates"]
                blueberry_type = item["blueberry_type"]

                assert blueberry_type in {
                    "Bluegold",
                    "Duke",
                    "Blueray",
                    "Other blueberry type",
                }, f"Unexpected blueberry type '{blueberry_type}' in data unit '{data_unit}'"

                if label_ref not in object_instances_by_label_ref:
                    polygon_object_instance: ObjectInstance = polygon_ontology_object.create_instance()
                    assert polygon_object_instance is not None, f"Failed to create ObjectInstance for {label_ref}"

                    object_instances_by_label_ref[label_ref] = polygon_object_instance
                    checklist_attribute = None

                    # Set blueberry type attribute
                    if blueberry_type == "Bluegold":
                        assert bluegold_option is not None, "Missing 'bluegold_option'"
                        polygon_object_instance.set_answer(
                            attribute=blueberry_type_radio_attribute, answer=bluegold_option
                        )
                        checklist_attribute = bluegold_checklist_attribute
                    elif blueberry_type == "Duke":
                        assert duke_option is not None, "Missing 'duke_option'"
                        polygon_object_instance.set_answer(attribute=blueberry_type_radio_attribute, answer=duke_option)
                        checklist_attribute = duke_checklist_attribute
                    elif blueberry_type == "Blueray":
                        assert blueray_option is not None, "Missing 'blueray_option'"
                        polygon_object_instance.set_answer(
                            attribute=blueberry_type_radio_attribute, answer=blueray_option
                        )
                        checklist_attribute = blueray_checklist_attribute
                    elif blueberry_type == "Other blueberry type":
                        assert other_blueberry_option is not None, "Missing 'other_blueberry_option'"
                        polygon_object_instance.set_answer(
                            attribute=blueberry_type_radio_attribute, answer=other_blueberry_option
                        )
                        text_answer = item.get("Specify blueberry type", "")
                        assert isinstance(text_answer, str), "'Specify blueberry type' must be a string"
                        polygon_object_instance.set_answer(
                            attribute=other_blueberry_option_text_attribute,
                            answer=text_answer,
                        )

                    # Set checklist attributes
                    checklist_answers = []
                    quality_key = f"{blueberry_type.lower()}_quality_options"
                    quality_options = item.get(quality_key, "").split(", ")

                    for quality in quality_options:
                        option = None
                        if quality == "Plump":
                            option = (
                                bluegold_plump_option
                                if blueberry_type == "Bluegold"
                                else duke_plump_option
                                if blueberry_type == "Duke"
                                else blueray_plump_option
                                if blueberry_type == "Blueray"
                                else None
                            )
                        elif quality == "Juicy":
                            option = (
                                bluegold_juicy_option
                                if blueberry_type == "Bluegold"
                                else duke_juicy_option
                                if blueberry_type == "Duke"
                                else blueray_juicy_option
                                if blueberry_type == "Blueray"
                                else None
                            )
                        elif quality == "Large":
                            option = (
                                bluegold_large_option
                                if blueberry_type == "Bluegold"
                                else duke_large_option
                                if blueberry_type == "Duke"
                                else blueray_large_option
                                if blueberry_type == "Blueray"
                                else None
                            )

                        if option:
                            checklist_answers.append(option)
                        else:
                            assert blueberry_type == "Other blueberry type", (
                                f"Invalid quality '{quality}' for type '{blueberry_type}'"
                            )

                    if checklist_attribute and checklist_answers:
                        polygon_object_instance.set_answer(
                            attribute=checklist_attribute, answer=checklist_answers, overwrite=True
                        )

                else:
                    polygon_object_instance = object_instances_by_label_ref[label_ref]

                # Assign the object to the frame and track it
                polygon_object_instance.set_for_frames(coordinates=coord, frames=frame_number)

        # Add object instances to label_row **only if they have frames assigned**
        for polygon_object_instance in object_instances_by_label_ref.values():
            assert isinstance(polygon_object_instance, ObjectInstance), "Expected ObjectInstance type"
            if polygon_object_instance.get_annotation_frames():
                label_row.add_object_instance(polygon_object_instance)

        # Save label row using the bundle
        assert label_row is not None, "Trying to save a None label row"
        label_row.save(bundle=bundle)

print("Labels with blueberry type radio buttons, checklist attributes, and text labels added for all data units.")



from datetime import datetime, timezone
from unittest.mock import MagicMock, patch

import pytest
from requests import Response, Session

from encord.configs import ENCORD_DOMAIN, SshConfig
from encord.exceptions import EncordException, SshKeyNotFound
from encord.http.querier import HEADER_CLOUD_TRACE_CONTEXT, Querier
from tests.fixtures import PRIVATE_KEY


@pytest.fixture
def querier():
    return Querier(config=SshConfig(PRIVATE_KEY))


@patch.object(Session, "send")
@patch("encord.exceptions.datetime", wraps=datetime)
def test_failed_http_request_prints_out_trace_id(dt: MagicMock, send: MagicMock, querier):
    fake_time = datetime(2023, 1, 1, 0, 0, 0, 0, tzinfo=timezone.utc)
    dt.now.return_value = fake_time

    res = Response()
    res.status_code = 500
    send.return_value = res

    try:
        querier.basic_getter(object)
        assert False, "Should never get here, previous line is expected to raise"
    except EncordException as e:
        trace_header = send.call_args_list[0].args[0].headers[HEADER_CLOUD_TRACE_CONTEXT]
        trace_id, span_id = trace_header.split(";")[0].split("/")

        assert (
            f"Error parsing JSON response:  timestamp='2023-01-01T00:00:00+00:00' trace_id='{trace_id}' span_id='{span_id}'"
            in str(e)
        )


@patch.object(Session, "send")
def test_response_error_message_including_domain(send: MagicMock, querier: Querier):
    res = Response()
    res.status_code = 400
    res._content = b'{"status":400,"response":["SSH_KEY_NOT_FOUND_ERROR"],"payload":"Your used SSH key does not exist. Please add this SSH key to your user profile."}'
    send.return_value = res

    with pytest.raises(SshKeyNotFound) as e_info:
        querier.basic_getter(object)
    assert ENCORD_DOMAIN in str(e_info.value)



from unittest.mock import MagicMock, patch

from encord import Project
from encord.common.time_parser import parse_datetime
from encord.http.v2.api_client import ApiClient
from encord.http.v2.payloads import Page
from encord.orm.analytics import (
    CollaboratorTimer,
    CollaboratorTimerParams,
    CollaboratorTimersGroupBy,
)
from encord.utilities.project_user import ProjectUserRole
from tests.fixtures import ontology, project, user_client

assert user_client and project and ontology

COLLABORATOR_TIMERS_PATH = "analytics/collaborators/timers"


def construct_timer(
    user_email="noone@nowhere.com", user_role=ProjectUserRole.ADMIN, data_title="data title 1", time_seconds=22.0
) -> CollaboratorTimer:
    return CollaboratorTimer(
        user_email=user_email,
        user_role=user_role,
        data_title=data_title,
        time_seconds=time_seconds,
    )


@patch.object(ApiClient, "get")
def test_project_collaborator_timers_empty_page(api_client_get: MagicMock, project: Project):
    after_time = parse_datetime("2023-01-01T21:00:00")

    api_client_get.return_value = Page[CollaboratorTimer](results=[])

    timers = list(project.list_collaborator_timers(after=after_time))

    api_client_get.assert_called_once()
    assert [] == timers


@patch.object(ApiClient, "get")
def test_project_collaborator_timers_single_page(api_client_get: MagicMock, project: Project):
    after_time = parse_datetime("2023-01-01T21:00:00")

    return_value = Page[CollaboratorTimer](
        results=[
            construct_timer(data_title="data title 1"),
            construct_timer(data_title="data title 2"),
        ]
    )

    api_client_get.return_value = return_value

    timers = list(project.list_collaborator_timers(after=after_time))
    api_client_get.assert_called_once_with(
        COLLABORATOR_TIMERS_PATH,
        params=CollaboratorTimerParams(
            project_hash=project.project_hash,
            after=after_time,
            before=None,
            group_by=CollaboratorTimersGroupBy.DATA_UNIT,
            page_size=100,
        ),
        result_type=Page[CollaboratorTimer],
        allow_none=False,
    )

    assert return_value.results == timers


@patch.object(ApiClient, "get")
def test_project_collaborator_timers_multi_page(api_client_get: MagicMock, project: Project):
    after_time = parse_datetime("2023-01-01T21:00:00")

    return_value_page_1 = Page[CollaboratorTimer](
        results=[
            construct_timer(data_title="data title 1"),
            construct_timer(data_title="data title 2"),
        ],
        next_page_token="page-token-1",
    )

    return_value_page_2 = Page[CollaboratorTimer](
        results=[
            construct_timer(data_title="data title 3"),
            construct_timer(data_title="data title 4"),
        ]
    )

    api_client_get.side_effect = [return_value_page_1, return_value_page_2]

    timers = list(project.list_collaborator_timers(after=after_time))

    assert api_client_get.call_count == 2
    assert timers == return_value_page_1.results + return_value_page_2.results

    # Check that the get method was called with the correct page token
    second_call_args, second_call_kwargs = api_client_get.call_args_list[1]
    assert second_call_kwargs["params"].page_token == "page-token-1"



"""
Code Block Name: Radio buttons
"""

# Import dependencies
import json
import os

from encord import EncordUserClient
from encord.objects import ObjectInstance
from encord.objects.attributes import Attribute, RadioAttribute

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
DATA_UNIT = "cherries-010.jpg"
OUTPUT_FILE_PATH = "/Users/chris-encord/radio_attribute_output.json"
BUNDLE_SIZE = 100

# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Get project and label rows
project = user_client.get_project(PROJECT_ID)
assert project is not None, f"Project with ID {PROJECT_ID} could not be loaded"

label_rows = project.list_label_rows_v2(data_title_eq=DATA_UNIT)
assert label_rows, f"No label rows found for data unit: {DATA_UNIT}"

# Initialize label rows using bundle
with project.create_bundle(bundle_size=BUNDLE_SIZE) as bundle:
    for label_row in label_rows:
        label_row.initialise_labels(bundle=bundle)

# Prepare data structure to collect all results
output_data = []


# Function to extract info from RadioAttribute (recursively if nested)
def extract_attributes(attribute: Attribute, object_instance: ObjectInstance, frame_number: int):
    frame_data = []

    if isinstance(attribute, RadioAttribute):
        attribute_answers = object_instance.get_answer(attribute)
        if not isinstance(attribute_answers, list):
            attribute_answers = [attribute_answers]

        for answer in attribute_answers:
            if answer is None:
                continue  # Skip empty answers

            answer_data = {
                "frame": frame_number + 1,
                "attribute_name": attribute.title,
                "attribute_hash": attribute.feature_node_hash,
                "answer_title": answer.title,
                "answer_hash": answer.feature_node_hash,
                "nested_attributes": [],
            }

            if hasattr(answer, "attributes") and answer.attributes:
                for nested_attribute in answer.attributes:
                    nested_data = extract_attributes(nested_attribute, object_instance, frame_number)
                    if nested_data:
                        answer_data["nested_attributes"].extend(nested_data)

            frame_data.append(answer_data)

    return frame_data


# Loop through label rows and extract attribute data
for label_row in label_rows:
    object_instances = label_row.get_object_instances()
    assert object_instances, f"No object instances found in label row {label_row.uid}"

    for object_instance in object_instances:
        annotations = object_instance.get_annotations()
        assert annotations, f"No annotations found for object instance {object_instance.object_hash}"

        ontology_item = object_instance.ontology_item
        assert ontology_item and ontology_item.attributes, (
            f"No ontology item or attributes found for object {object_instance.object_hash}"
        )

        for annotation in annotations:
            for attribute in ontology_item.attributes:
                attr_data = extract_attributes(attribute, object_instance, annotation.frame)
                if attr_data:
                    output_data.extend(attr_data)

# Save results to JSON
assert OUTPUT_FILE_PATH.endswith(".json"), "Output file path must end with .json"

with open(OUTPUT_FILE_PATH, "w") as f:
    json.dump(output_data, f, indent=4)

print(f"Radio attribute data saved to {OUTPUT_FILE_PATH}")



"""
Code Block Name: Text attributes
"""

# Import dependencies
import json

from encord import EncordUserClient
from encord.objects import ObjectInstance
from encord.objects.attributes import Attribute, TextAttribute

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
DATA_UNIT = "cherries-is"
OUTPUT_FILE_PATH = "/Users/chris-encord/text_attributes_output.json"
BUNDLE_SIZE = 100

# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Specify Project
project = user_client.get_project(PROJECT_ID)
assert project is not None, f"Project with ID {PROJECT_ID} could not be loaded"

# Filter label rows for a specific data title
label_rows = project.list_label_rows_v2(data_title_eq=DATA_UNIT)
assert label_rows, f"No label rows found for data title '{DATA_UNIT}'"

# Initialize labels using bundle
with project.create_bundle(bundle_size=BUNDLE_SIZE) as bundle:
    for label_row in label_rows:
        label_row.initialise_labels(bundle=bundle)


# Function to extract text attributes and store in structured format
def extract_text_attributes(attribute: Attribute, object_instance: ObjectInstance, frame_number: int):
    if isinstance(attribute, TextAttribute):
        text_answer = object_instance.get_answer(attribute)
        return {
            "frame": frame_number + 1,
            "attribute_name": attribute.title,
            "attribute_hash": attribute.feature_node_hash,
            "text_answer": text_answer,
        }
    return None


# Collect results for saving
results = []

# Iterate through all object instances and collect text attribute data
for label_row in label_rows:
    object_instances = label_row.get_object_instances()
    assert object_instances, f"No object instances found in label row {label_row.uid}"

    for object_instance in object_instances:
        annotations = object_instance.get_annotations()
        assert annotations, f"No annotations found for object instance {object_instance.object_hash}"

        ontology_item = object_instance.ontology_item
        assert ontology_item and ontology_item.attributes, (
            f"Missing ontology item or attributes for object {object_instance.object_hash}"
        )

        for annotation in annotations:
            for attribute in ontology_item.attributes:
                attr_data = extract_text_attributes(attribute, object_instance, annotation.frame)
                if attr_data:
                    results.append(attr_data)
                    # Optional: also print to console
                    print(f"Frame {attr_data['frame']}:")
                    print(f"Text Attribute name: {attr_data['attribute_name']}")
                    print(f"Text Attribute hash: {attr_data['attribute_hash']}")
                    print(f"Text Attribute Answer: {attr_data['text_answer']}")

# Save results to JSON
assert OUTPUT_FILE_PATH.endswith(".json"), "Output file path must end with .json"

with open(OUTPUT_FILE_PATH, "w") as f:
    json.dump(results, f, indent=4)

print(f"\nText attribute data saved to: {OUTPUT_FILE_PATH}")



import pytest

from encord.objects.frames import Range, frames_class_to_frames_list


def test_frames_class_to_frames_list():
    # Single frame
    assert frames_class_to_frames_list(1) == [1]
    assert frames_class_to_frames_list(10) == [10]

    # Range
    assert frames_class_to_frames_list(Range(1, 4)) == [1, 2, 3, 4]
    assert frames_class_to_frames_list(Range(0, 1)) == [0, 1]
    assert frames_class_to_frames_list(Range(4, 4)) == [4]

    # List of Range
    list_of_range_1 = [Range(2, 3), Range(3, 5), Range(7, 7)]
    assert frames_class_to_frames_list(list_of_range_1) == [2, 3, 4, 5, 7]
    list_of_range_2 = [Range(10, 12), Range(3, 5), Range(8, 9), Range(8, 9)]
    assert frames_class_to_frames_list(list_of_range_2) == [3, 4, 5, 8, 9, 10, 11, 12]

    # List of integer frame numbers
    # Empty List
    assert frames_class_to_frames_list([]) == []
    list_of_integers_1 = [4, 5, 6, 24, 60]
    assert frames_class_to_frames_list(list_of_integers_1) == list_of_integers_1
    assert frames_class_to_frames_list([4]) == [4]
    assert frames_class_to_frames_list([9, 8, 7]) == [7, 8, 9]
    assert frames_class_to_frames_list([9, 2, 4, 4, 14]) == [2, 4, 9, 14]

    # Exception
    with pytest.raises(RuntimeError):
        frames_class_to_frames_list([4, Range(4, 6)])
    with pytest.raises(RuntimeError):
        frames_class_to_frames_list({3, 5, 7})



import numpy as np

from encord.common.bitmask_operations.bitmask_operations import (
    _mask_to_rle,
    _rle_to_mask,
    _rle_to_string,
    _string_to_rle,
    serialise_bitmask,
    transpose_bytearray,
)
from encord.objects.bitmask import (
    BitmaskCoordinates,
)


def test_rle_decode():
    resolution = 1914 * 2294

    encoded_str = "V[^i0g1Rj1101Df0D2N2G?H20OJ;J2O001I<J000J<J0O2J:J101J7M2O0N4M2N1L6M2OOK<I20ON4L3O000J9M001I8N0O1N201N1K7M2O0N4M2O0K7M2OOL6N100N4M2O0O1K8L100J60001I8N0O1N2O101J7M100K5O100N2O10001I8N0O1N200O1K5O2O0K7M2N001O2N0L501OOO3N10OK8M3N000J60000M211N1K402OOO2O2OOL5O2OOL6N10OO3N100K5O100O1N102O0J511OOK601N0O201N0L5O2OOO3N11NN1110OL6N011O0M211N0L3110OO2O2OOL5O2OOO2O2N00O1010K5O100O0O020O1K5O10OO10010M300O0L320O0O3O000J32100NO21000NO1200OK420O1N201N0L31010N01010NO2100KO5100NO2011NOM3110OO10010NM401OL041O00L41O0L05O10NL421OO0NN500OK06000M12O01NM5O00OM410OL0411NN11010NL51000OK150O0OL61N0O0110OL04010NL51O1ON2100KO510OOM4010O0O0110OL0410OO0110OOM41O0L05O01N02O10NL421OK212000ONM600ONO4O00L4001OOM410OO0110OL04010NL51000ON120O0LO61N0O0110OOM4010K31001ON2100NL5100K22100OO0M410OO0110OO1001OOM41O0L320O0OM5O10N0012ONO12000OK1500ON21O00OM50O0O0110OO01010KO5100NO21000NON51ONO120O1K31010NM401OO0110OOM411NN1110OL040100ON120O0O0110OOM410OL311O00O11O00L4010NO2100NO2100O0LO52OOOO22OOO0110OOM4001K32O01NO211ONL421NO001100ON1200ONO4O00O11O00O020O0OM50O0O0102ON001100ONN500OK4200ON120O0OM410OO01010N0102OMM5100NO21000ON031OONM60O1N02O01N01010NL5100NO211NK4110OO010100ONN50O0O11O00O1001OOM410OO0110OO01010N010100ONN500ON120O0O020O0O0110OO0110OOM402OON3000NO21000ON1200ONM61N0O020O0O0110OO01010N0102OMM5100NO21000NO121OO1M30O1NM5O00O1001OO0111N1N111NN1111NOM31100O1N200ON21O1O2N001OOM410O1N20O00O10100N2O100ON1200O1N20O0O1002N1K411N1N111N1N2O10M1102OON4O00O02M110O2M110OO0111N1N111N1N111N0O2O2OON30000M30000O2J410O2M111N1N111N1N111N1N102OON301OON30000O1N200O2L220N3L201N1N111N1N102O0N102O2K4O00ON4O00O02M201O2M11OO1N200O3L3O0O1N200O4K2O100M30003J30000000001N4K111N3L201N3L201N10O20N3L3N102L3N103J30001N3L201N2J6O1N6I201N3L201N3L3N106E5O106D60002M6E610M7F501N8B8N2N<ZO;01NgRhi1"

    r = _string_to_rle(encoded_str)
    assert sum(r) == resolution

    re_encoded_str = _rle_to_string(r)
    assert encoded_str == re_encoded_str


def test_basic_rle_to_mask():
    res = _rle_to_mask([5, 2, 5], size=12)
    assert res == b"\00\00\00\00\00\01\01\00\00\00\00\00"


def test_basic_rle_to_mask_zebra():
    res = _rle_to_mask([1, 1, 1, 1, 1, 1, 1, 1], size=8)
    assert res == b"\00\01\00\01\00\01\00\01"


def test_basic_mask_to_rle():
    mask = b"\00\00\00\00\00\01\01\00\00\00\00\00"
    rle = _mask_to_rle(mask)
    assert rle == [5, 2, 5]


def test_basic_mask_to_rle_zebra():
    mask = b"\00\01\00\01\00\01\00\01"
    rle = _mask_to_rle(mask)
    assert rle == [1, 1, 1, 1, 1, 1, 1, 1]


def test_rle_to_mask():
    resolution = 1914 * 2294
    encoded_str = "V[^i0g1Rj1101Df0D2N2G?H20OJ;J2O001I<J000J<J0O2J:J101J7M2O0N4M2N1L6M2OOK<I20ON4L3O000J9M001I8N0O1N201N1K7M2O0N4M2O0K7M2OOL6N100N4M2O0O1K8L100J60001I8N0O1N2O101J7M100K5O100N2O10001I8N0O1N200O1K5O2O0K7M2N001O2N0L501OOO3N10OK8M3N000J60000M211N1K402OOO2O2OOL5O2OOL6N10OO3N100K5O100O1N102O0J511OOK601N0O201N0L5O2OOO3N11NN1110OL6N011O0M211N0L3110OO2O2OOL5O2OOO2O2N00O1010K5O100O0O020O1K5O10OO10010M300O0L320O0O3O000J32100NO21000NO1200OK420O1N201N0L31010N01010NO2100KO5100NO2011NOM3110OO10010NM401OL041O00L41O0L05O10NL421OO0NN500OK06000M12O01NM5O00OM410OL0411NN11010NL51000OK150O0OL61N0O0110OL04010NL51O1ON2100KO510OOM4010O0O0110OL0410OO0110OOM41O0L05O01N02O10NL421OK212000ONM600ONO4O00L4001OOM410OO0110OL04010NL51000ON120O0LO61N0O0110OOM4010K31001ON2100NL5100K22100OO0M410OO0110OO1001OOM41O0L320O0OM5O10N0012ONO12000OK1500ON21O00OM50O0O0110OO01010KO5100NO21000NON51ONO120O1K31010NM401OO0110OOM411NN1110OL040100ON120O0O0110OOM410OL311O00O11O00L4010NO2100NO2100O0LO52OOOO22OOO0110OOM4001K32O01NO211ONL421NO001100ON1200ONO4O00O11O00O020O0OM50O0O0102ON001100ONN500OK4200ON120O0OM410OO01010N0102OMM5100NO21000ON031OONM60O1N02O01N01010NL5100NO211NK4110OO010100ONN50O0O11O00O1001OOM410OO0110OO01010N010100ONN500ON120O0O020O0O0110OO0110OOM402OON3000NO21000ON1200ONM61N0O020O0O0110OO01010N0102OMM5100NO21000NO121OO1M30O1NM5O00O1001OO0111N1N111NN1111NOM31100O1N200ON21O1O2N001OOM410O1N20O00O10100N2O100ON1200O1N20O0O1002N1K411N1N111N1N2O10M1102OON4O00O02M110O2M110OO0111N1N111N1N111N0O2O2OON30000M30000O2J410O2M111N1N111N1N111N1N102OON301OON30000O1N200O2L220N3L201N1N111N1N102O0N102O2K4O00ON4O00O02M201O2M11OO1N200O3L3O0O1N200O4K2O100M30003J30000000001N4K111N3L201N3L201N10O20N3L3N102L3N103J30001N3L201N2J6O1N6I201N3L201N3L3N106E5O106D60002M6E610M7F501N8B8N2N<ZO;01NgRhi1"
    rle = _string_to_rle(encoded_str)
    mask = _rle_to_mask(rle, size=resolution)
    assert len(mask) == resolution


def test_mask_roundtrip():
    orig_mask = np.zeros((2000, 3000), dtype=bool)
    orig_mask[:1000, 1000:] = 1
    orig_mask[1000:, :1000] = 1

    mask_coordinates = BitmaskCoordinates(orig_mask)

    new_mask = mask_coordinates.to_numpy_array()
    assert np.allclose(orig_mask, new_mask)

    new_mask_using_constructor = np.array(mask_coordinates)
    assert np.allclose(orig_mask, new_mask_using_constructor)


def test_mask_rle_mask_roundtrip():
    orig_mask = np.zeros((2000, 3000), dtype=bool)
    orig_mask[:1000, 1000:] = 1
    orig_mask[1000:, :1000] = 1

    mask_coordinates = BitmaskCoordinates(orig_mask)
    mask_dict = mask_coordinates.to_dict()

    new_mask = BitmaskCoordinates.from_dict(mask_dict).to_numpy_array()
    assert np.allclose(orig_mask, new_mask)


def test_transpose_bytearray():
    mask = b"\00\00\00\00\00\01\01\00\00\00\00\00"
    shape = (3, 4)
    expected = b"\x00\x00\x00\x00\x01\x00\x00\x01\x00\x00\x00\x00"
    ans = transpose_bytearray(mask, shape)
    assert ans == expected


def test_serialise_bitmask_starting_with_true():
    mask = np.array(
        [
            [True, True, True],
            [False, False, False],
        ]
    )
    shape = mask.shape
    rle_string = serialise_bitmask(mask.tobytes())
    mask_encoded = BitmaskCoordinates.EncodedBitmask(
        top=0, left=0, height=shape[0], width=shape[1], rle_string=rle_string
    )
    mask_decoded = BitmaskCoordinates(mask_encoded).to_numpy_array()

    assert np.array_equal(mask, mask_decoded)


def test_serialise_bitmask_empty():
    mask = np.array([[]], dtype=bool)
    shape = mask.shape

    rle_string = serialise_bitmask(mask.tobytes())
    mask_encoded = BitmaskCoordinates.EncodedBitmask(
        top=0, left=0, height=shape[0], width=shape[1], rle_string=rle_string
    )
    mask_decoded = BitmaskCoordinates(mask_encoded).to_numpy_array()

    assert np.array_equal(mask, mask_decoded)  # Mask is inverted and the test fails.



from typing import Dict, Optional

from pydantic import Field

from encord.orm.base_dto import BaseDTO


class SkeletonTemplateCoordinate(BaseDTO):
    x: float
    y: float
    name: str
    color: Optional[str] = "#00000"
    value: Optional[str] = ""
    feature_hash: Optional[str] = None


class SkeletonTemplate(BaseDTO):
    name: str
    width: float
    height: float
    skeleton: Dict[str, SkeletonTemplateCoordinate]
    skeleton_edges: Dict[str, Dict[str, Dict[str, str]]]  # start-end-color-hex
    feature_node_hash: Optional[str] = Field(default=None, alias="feature_node_hash")
    shape: Optional[str] = "skeleton"



import datetime

from encord.orm.label_row import AnnotationTaskStatus, LabelRowMetadata, LabelStatus

FAKE_LABEL_ROW_METADATA = LabelRowMetadata(
    label_hash="",
    branch_name="main",
    created_at=datetime.datetime.now(),
    last_edited_at=datetime.datetime.now(),
    data_hash="",
    data_title="",
    data_type="VIDEO",
    data_link="",
    dataset_hash="",
    dataset_title="",
    label_status=LabelStatus.NOT_LABELLED,
    annotation_task_status=AnnotationTaskStatus.QUEUED,
    workflow_graph_node=None,
    is_shadow_data=False,
    duration=100,
    frames_per_second=25,
    number_of_frames=100 * 25,
    height=100,
    width=10,
    audio_codec=None,
    audio_bit_depth=None,
    audio_num_channels=None,
    audio_sample_rate=None,
)



ontology = {
    "objects": [
        {
            "id": "1",
            "name": "box",
            "color": "#D33115",
            "shape": "bounding_box",
            "featureNodeHash": "LJNV5PVK",
            "attributes": [
                {
                    "id": "1.1",
                    "name": "dynamic text",
                    "type": "text",
                    "featureNodeHash": "5RGh3YIU",
                    "required": False,
                    "dynamic": True,
                },
                {
                    "id": "1.2",
                    "name": "dynamic classification",
                    "type": "checklist",
                    "featureNodeHash": "ZJoiBceM",
                    "required": False,
                    "dynamic": True,
                    "options": [
                        {
                            "id": "1.2.1",
                            "label": "classification 1",
                            "value": "classification_1",
                            "featureNodeHash": "aOQGJvce",
                        },
                        {
                            "id": "1.2.2",
                            "label": "classification 2",
                            "value": "classification_2",
                            "featureNodeHash": "tyjScPrk",
                        },
                    ],
                },
                {
                    "id": "1.3",
                    "name": "dynamic radio",
                    "type": "radio",
                    "featureNodeHash": "OMpf1zPf",
                    "required": False,
                    "dynamic": True,
                    "options": [
                        {"id": "1.3.1", "label": "radio 1", "value": "radio_1", "featureNodeHash": "ps0dwXfI"},
                        {"id": "1.3.2", "label": "radio 2", "value": "radio_2", "featureNodeHash": "IwCHSdeL"},
                    ],
                },
            ],
        },
        {
            "id": "2",
            "name": "box 2",
            "color": "#E27300",
            "shape": "bounding_box",
            "featureNodeHash": "GaAKWvvg",
            "attributes": [
                {
                    "id": "2.1",
                    "name": "radio 2",
                    "type": "radio",
                    "featureNodeHash": "TWPByH4G",
                    "required": False,
                    "dynamic": True,
                    "options": [
                        {"id": "2.1.1", "label": "radio 2 1", "value": "radio_2_1", "featureNodeHash": "zhrqAYjQ"},
                        {"id": "2.1.2", "label": "radio 2 2", "value": "radio_2_2", "featureNodeHash": "J/I/karX"},
                    ],
                }
            ],
        },
    ],
    "classifications": [],
}



from unittest.mock import MagicMock, patch

import pytest
from requests import Response, Session

from encord.configs import ENCORD_DOMAIN, SshConfig
from encord.exceptions import (
    AuthenticationError,
    AuthorisationError,
    EncordException,
    InvalidArgumentsError,
    ResourceNotFoundError,
    SshKeyNotFound,
    UnknownException,
)
from encord.http.v2.api_client import ApiClient
from encord.orm.analytics import CollaboratorTimer
from tests.fixtures import PRIVATE_KEY


@pytest.fixture
def api_client():
    return ApiClient(config=SshConfig(PRIVATE_KEY))


@patch.object(Session, "send")
def test_response_mapping_status_codes_to_exception_type_500(send: MagicMock, api_client: ApiClient):
    res = Response()
    res.status_code = 500
    send.return_value = res

    with pytest.raises(UnknownException):
        api_client.get("/", params=None, result_type=CollaboratorTimer)


@patch.object(Session, "send")
def test_response_mapping_status_codes_to_exception_type_401(send: MagicMock, api_client: ApiClient):
    res = Response()
    res.status_code = 401
    send.return_value = res

    with pytest.raises(AuthenticationError):
        api_client.get("/", params=None, result_type=CollaboratorTimer)


@patch.object(Session, "send")
def test_response_mapping_status_codes_to_exception_type_403(send: MagicMock, api_client: ApiClient):
    res = Response()
    res.status_code = 403
    send.return_value = res

    with pytest.raises(AuthorisationError):
        api_client.get("/", params=None, result_type=CollaboratorTimer)


@patch.object(Session, "send")
def test_response_mapping_status_codes_to_exception_type_404(send: MagicMock, api_client: ApiClient):
    res = Response()
    res.status_code = 404
    send.return_value = res

    with pytest.raises(ResourceNotFoundError):
        api_client.get("/", params=None, result_type=CollaboratorTimer)


@patch.object(Session, "send")
def test_response_mapping_status_codes_to_exception_type_unknown(send: MagicMock, api_client: ApiClient):
    res = Response()
    res.status_code = 66
    send.return_value = res

    with pytest.raises(UnknownException):
        api_client.get("/", params=None, result_type=CollaboratorTimer)


@patch.object(Session, "send")
def test_response_error_message_including_domain(send: MagicMock, api_client: ApiClient):
    res = Response()
    res.status_code = 400
    send.return_value = res

    with pytest.raises(EncordException) as e_info:
        api_client.get("/", params=None, result_type=CollaboratorTimer)
    assert ENCORD_DOMAIN in str(e_info.value)



import contextlib

import pytest

from encord.common.utils import validate_user_agent_suffix

valid_examples = [
    "CERN-LineMode/2.15 libwww/2.17b3",
    "Mozilla/5.0",
    "Simple-Bot",
    "Product/1.0 AnotherProduct/2.0",
    "Chrome/91.0.4472.124 Safari/537.36",
]

invalid_examples = [
    "",  # Empty string
    "/",  # Just a slash
    "Invalid/Version/",  # Trailing slash
    "Product/1.0/",  # Trailing slash
    "Product//1.0",  # Double slash
    "/1.0",  # Missing product name
]


@pytest.mark.parametrize(
    ["user_agent_suffix", "should_raise"],
    [(example, False) for example in valid_examples] + [(example, True) for example in invalid_examples],
)
def test_user_agent_validate_happy(user_agent_suffix: str, should_raise: bool) -> None:
    context = pytest.raises(ValueError) if should_raise else contextlib.nullcontext()
    with context:
        validate_user_agent_suffix(user_agent_suffix)



from __future__ import annotations

from dataclasses import dataclass
from enum import Enum
from typing import Iterable, List, Literal, Optional, Union
from uuid import UUID

from encord.common.utils import ensure_list, ensure_uuid_list
from encord.http.bundle import Bundle
from encord.orm.workflow import WorkflowStageType
from encord.workflow.common import TasksQueryParams, WorkflowAction, WorkflowStageBase, WorkflowTask


class AgentTaskStatus(str, Enum):
    NEW = "NEW"
    ASSIGNED = "ASSIGNED"
    RELEASED = "RELEASED"
    SKIPPED = "SKIPPED"
    REOPENED = "REOPENED"
    COMPLETED = "COMPLETED"


class _ActionPathway(WorkflowAction):
    action: Literal["PATHWAY_ACTION"] = "PATHWAY_ACTION"
    pathway_uuid: Optional[Union[str, UUID]] = None
    pathway_name: Optional[str] = None


class AgentTask(WorkflowTask):
    """Represents a task in an Agent stage.

    **Attributes**

    - `status` (AgentTaskStatus): The current status of the task.
    - `data_hash` (UUID): Unique ID for the data unit.
    - `data_title` (str): Name of the data unit.
    - `label_branch_name` (str): Name of the label branch.
    - `assignee` (Optional[str]): User assigned to the task.
    - `last_actioned_by` (Optional[str]): User who last acted on the task.

    **Allowed actions**

    ...
    """

    status: AgentTaskStatus
    data_hash: UUID
    data_title: str
    label_branch_name: str
    assignee: Optional[str]
    last_actioned_by: Optional[str]

    def proceed(
        self,
        pathway_name: Optional[str] = None,
        pathway_uuid: Optional[Union[UUID, str]] = None,
        *,
        bundle: Optional[Bundle] = None,
    ) -> None:
        if not pathway_name and not pathway_uuid:
            raise ValueError("Either `pathway_name` or `pathway_uuid` parameter must be provided.")

        workflow_client, stage_uuid = self._get_client_data()
        workflow_client.action(
            stage_uuid,
            _ActionPathway(
                task_uuid=self.uuid,
                pathway_name=pathway_name,
                pathway_uuid=pathway_uuid,
            ),
            bundle=bundle,
        )


class _AgentTasksQueryParams(TasksQueryParams):
    user_emails: Optional[List[str]] = None
    data_hashes: Optional[List[UUID]] = None
    dataset_hashes: Optional[List[UUID]] = None
    data_title_contains: Optional[str] = None
    statuses: Optional[List[AgentTaskStatus]] = None


@dataclass(frozen=True)
class AgentPathway:
    uuid: UUID
    name: str
    destination_uuid: UUID


@dataclass(frozen=True)
class AgentStage(WorkflowStageBase):
    pathways: list[AgentPathway]
    stage_type: Literal[WorkflowStageType.AGENT] = WorkflowStageType.AGENT

    """
    An Agent stage.
    """

    def get_tasks(
        self,
        *,
        assignee: Union[List[str], str, None] = None,
        data_hash: Union[List[UUID], UUID, List[str], str, None] = None,
        dataset_hash: Union[List[UUID], UUID, List[str], str, None] = None,
        data_title: Optional[str] = None,
        status: Optional[AgentTaskStatus | List[AgentTaskStatus]] = None,
    ) -> Iterable[AgentTask]:
        """Retrieves tasks for the AgentStage.

        **Parameters**

        - `assignee` (Union[List[str], str, None]): A list of user emails or a single user email to filter tasks by assignee.
        - `data_hash` (Union[List[UUID], UUID, List[str], str, None]): A list of data unit UUIDs or a single data unit UUID to filter tasks by data hash.
        - `dataset_hash` (Union[List[UUID], UUID, List[str], str, None]): A list of dataset UUIDs or a single dataset UUID to filter tasks by dataset hash.
        - `data_title` (Optional[str]): A string to filter tasks by data title.
        - `status` (Optional[AnnotationTaskStatus | List[AnnotationTaskStatus]]): A status or a list of statuses to filter tasks by their status.

        **Returns**

        An iterable of `AnnotationTask` instances from both non-Consensus and Consensus Projects.
        """
        params = _AgentTasksQueryParams(
            user_emails=ensure_list(assignee),
            data_hashes=ensure_uuid_list(data_hash),
            dataset_hashes=ensure_uuid_list(dataset_hash),
            data_title_contains=data_title,
            statuses=ensure_list(status),
        )

        for task in self._workflow_client.get_tasks(self.uuid, params, type_=AgentTask):
            task._stage_uuid = self.uuid
            task._workflow_client = self._workflow_client
            yield task



from copy import deepcopy
from typing import Dict
from unittest.mock import MagicMock, patch

from encord import Project
from encord.client import EncordClientProject
from encord.objects import LabelRowV2
from encord.orm.label_row import LabelRow, LabelRowMetadata
from tests.fixtures import ontology, project, user_client
from tests.test_data.label_rows_metadata_blurb import (
    LABEL_ROW_BLURB,
    LABEL_ROW_METADATA_BLURB,
)

assert user_client and project and ontology


def remove_label_hash(obj: Dict) -> Dict:
    obj = deepcopy(obj)
    del obj["label_hash"]
    return obj


def get_response_by_data_hash(data_hash: str):
    for r in LABEL_ROW_BLURB:
        if r["data_hash"] == data_hash:
            return r
    assert False


def get_valid_label_rows(project: Project):
    label_rows = []
    for r in LABEL_ROW_METADATA_BLURB:
        label_rows.append(LabelRowV2(LabelRowMetadata.from_dict(r), project._client, project._ontology))

    for r, v in zip(label_rows, LABEL_ROW_BLURB):
        assert r.data_hash == v["data_hash"]
        r.from_labels_dict(v)

    return label_rows


@patch.object(EncordClientProject, "get_label_rows")
@patch.object(EncordClientProject, "create_label_rows")
@patch.object(EncordClientProject, "list_label_rows")
def test_bundled_label_initialise_create(
    list_label_rows_mock: MagicMock, create_label_rows_mock: MagicMock, get_label_rows_mock: MagicMock, project: Project
):
    list_label_rows_mock.return_value = [
        LabelRowMetadata.from_dict(remove_label_hash(row)) for row in LABEL_ROW_METADATA_BLURB
    ]
    create_label_rows_mock.return_value = [LabelRow(row) for row in LABEL_ROW_BLURB]

    rows = project.list_label_rows_v2()

    bundle = project.create_bundle()
    for row in rows:
        row.initialise_labels(bundle=bundle)

    # making sure not calls were made at this point
    get_label_rows_mock.assert_not_called()

    bundle.execute()

    create_label_rows_mock.assert_called_once()
    get_label_rows_mock.assert_not_called()

    args = create_label_rows_mock.call_args[1]
    assert args is not None
    assert len(args["uids"]) == 3, "Expected 3 requests bundled"

    for row in rows:
        assert row.is_labelling_initialised, "Expect all rows to be intitialised"


@patch.object(EncordClientProject, "get_label_rows")
@patch.object(EncordClientProject, "create_label_rows")
@patch.object(EncordClientProject, "list_label_rows")
def test_bundled_label_initialise_get(
    list_label_rows_mock: MagicMock, create_label_rows_mock: MagicMock, get_label_rows_mock: MagicMock, project: Project
):
    list_label_rows_mock.return_value = [LabelRowMetadata.from_dict(row) for row in LABEL_ROW_METADATA_BLURB]
    get_label_rows_mock.return_value = [LabelRow(row) for row in LABEL_ROW_BLURB]

    rows = project.list_label_rows_v2()

    bundle = project.create_bundle()
    for row in rows:
        row.initialise_labels(bundle=bundle)

    # making sure not calls were made at this point
    get_label_rows_mock.assert_not_called()

    bundle.execute()

    create_label_rows_mock.assert_not_called()
    get_label_rows_mock.assert_called_once()

    args = get_label_rows_mock.call_args[1]
    assert args is not None
    assert len(args["uids"]) == 3, "Expected 3 requests bundled"

    for row in rows:
        assert row.is_labelling_initialised, "Expect all rows to be initialized"


@patch.object(EncordClientProject, "get_label_rows")
@patch.object(EncordClientProject, "create_label_rows")
@patch.object(EncordClientProject, "list_label_rows")
def test_bundled_label_initialise_mix_get_create(
    list_label_rows_mock: MagicMock, create_label_rows_mock: MagicMock, get_label_rows_mock: MagicMock, project: Project
):
    responses = [
        get_response_by_data_hash(data_hash) for data_hash in [row["data_hash"] for row in LABEL_ROW_METADATA_BLURB]
    ]

    rows_metadata_mix = LABEL_ROW_METADATA_BLURB[:2] + [remove_label_hash(row) for row in LABEL_ROW_METADATA_BLURB[2:]]
    list_label_rows_mock.return_value = [LabelRowMetadata.from_dict(row) for row in rows_metadata_mix]

    get_label_rows_mock.return_value = [LabelRow(row) for row in responses[:2]]
    create_label_rows_mock.return_value = [LabelRow(row) for row in responses[2:]]

    rows = project.list_label_rows_v2()

    bundle = project.create_bundle()
    for row in rows:
        row.initialise_labels(bundle=bundle)

    # making sure not calls were made at this point
    get_label_rows_mock.assert_not_called()

    bundle.execute()

    create_label_rows_mock.assert_called_once()
    get_label_rows_mock.assert_called_once()

    for row in rows:
        assert row.is_labelling_initialised, "Expect all rows to be intitialised"


@patch.object(EncordClientProject, "save_label_rows")
def test_bundled_label_save(save_label_rows_mock: MagicMock, project: Project):
    label_rows = get_valid_label_rows(project)

    bundle = project.create_bundle()
    for row in label_rows:
        row.save(bundle=bundle)

    save_label_rows_mock.assert_not_called()

    bundle.execute()

    save_label_rows_mock.assert_called_once()

    args = save_label_rows_mock.call_args[1]
    assert args is not None
    assert len(args["uids"]) == 3, "Expected 3 updates bundled"
    assert len(args["payload"]) == 3, "Expected 3 updates bundled"


@patch.object(EncordClientProject, "save_label_rows")
def test_bundled_label_save_with_explicit_bundle_size(save_label_rows_mock: MagicMock, project: Project):
    label_rows = get_valid_label_rows(project)
    assert len(label_rows) == 3

    bundle = project.create_bundle(bundle_size=2)
    for row in label_rows:
        row.save(bundle=bundle)

    save_label_rows_mock.assert_not_called()

    bundle.execute()

    assert save_label_rows_mock.call_count == 2

    args_0 = save_label_rows_mock.call_args_list[0][1]
    assert args_0 is not None
    assert len(args_0["uids"]) == 2, "Expected 2 updates bundled in the first bundle"
    assert len(args_0["payload"]) == 2, "Expected 2 updates bundled in the fist bundle"

    args_1 = save_label_rows_mock.call_args_list[1][1]
    assert args_1 is not None
    assert len(args_1["uids"]) == 1, "Expected 1 updates bundled in the first bundle"
    assert len(args_1["payload"]) == 1, "Expected 1 updates bundled in the fist bundle"



from encord.constants.enums import DataType


def test_data_type_consistency() -> None:
    values_from_string = set()
    for value in DataType:
        string_representation = value.to_upper_case_string()
        value_from_string = DataType.from_upper_case_string(string_representation)
        values_from_string.add(value_from_string)
        assert value == value_from_string

    assert len(values_from_string) == len(DataType)


def test_data_type_missing() -> None:
    assert DataType.from_upper_case_string("new_cord_type").value == "_MISSING_DATA_TYPE_"
    assert DataType.from_upper_case_string("new_cord_type") == "_MISSING_DATA_TYPE_"
    assert DataType("new_cord_type").value == "_MISSING_DATA_TYPE_"
    assert DataType("new_cord_type") == "_MISSING_DATA_TYPE_"



import uuid
from datetime import datetime, timezone
from typing import List

import pytest

from encord.common.time_parser import format_datetime_to_long_string
from encord.exceptions import EncordException
from encord.orm.base_dto import BaseDTO, BaseDTOWithExtra, dto_validator
from encord.orm.dataset import DatasetDataLongPolling, LongPollingStatus


class TestModel(BaseDTO):
    text_value: str
    number_value: int
    datetime_value: datetime


class TestModelWithLists(BaseDTO):
    text_values: List[str]
    number_values: List[int]
    datetime_values: List[datetime]


class TestModelWithExtra(BaseDTOWithExtra):
    text_value: str


def test_deserialize_model_with_extra():
    datetime_value = datetime.now()
    data_dict = {
        "text_value": "abc",
        "int_value": 22,
        "float_value": 22.22,
        "datetime_value": format_datetime_to_long_string(datetime_value),
    }

    model = TestModelWithExtra.from_dict(data_dict)

    assert model.text_value == "abc"
    assert model.get_extra("int_value") == 22
    assert model.get_extra("float_value") == 22.22
    assert isinstance(model.get_extra("datetime_value"), str)


def test_basic_model_with_deserialization():
    datetime_value = datetime.now()
    data_dict = {
        "text_value": "abc",
        "number_value": 22,
        "datetime_value": format_datetime_to_long_string(datetime_value),
    }

    model = TestModel.from_dict(data_dict)

    assert model.text_value == "abc"
    assert model.number_value == 22
    assert model.datetime_value == datetime_value.replace(microsecond=0).astimezone(tz=timezone.utc)


def test_basic_model_with_naive_iso_datetime_deserialization():
    datetime_value = datetime.now()
    data_dict = {"text_value": "abc", "number_value": 22, "datetime_value": str(datetime_value)}

    model = TestModel.from_dict(data_dict)

    assert model.text_value == "abc"
    assert model.number_value == 22
    assert model.datetime_value == datetime_value.replace(tzinfo=timezone.utc)


def test_basic_model_with_value_containers_deserialization():
    data_dict = {
        "text_values": ["abc", "def"],
        "number_values": [22, 33],
        "datetime_values": [
            str(datetime(year=2024, month=2, day=1, tzinfo=timezone.utc)),
            str(datetime(year=2024, month=2, day=2, tzinfo=timezone.utc)),
        ],
    }
    model = TestModelWithLists.from_dict(data_dict)
    assert model.text_values == ["abc", "def"]
    assert model.number_values == [22, 33]
    assert model.datetime_values == [
        datetime(year=2024, month=2, day=1, tzinfo=timezone.utc),
        datetime(year=2024, month=2, day=2, tzinfo=timezone.utc),
    ]


def test_complex_model_deserialization():
    backing_item_uuid = uuid.uuid4()
    data_dict = {
        "status": "PENDING",
        "errors": ["error one", "error two"],
        "units_pending_count": 5,
        "units_done_count": 5,
        "units_error_count": 2,
        "units_cancelled_count": 0,
        "data_hashes_with_titles": [
            {"data_hash": "abc", "title": "dummy title", "backing_item_uuid": str(backing_item_uuid)}
        ],
        "data_unit_errors": [],
    }

    model = DatasetDataLongPolling.from_dict(data_dict)
    assert model.status == LongPollingStatus.PENDING
    assert model.errors == ["error one", "error two"]
    assert model.data_hashes_with_titles[0].data_hash == "abc"
    assert model.data_hashes_with_titles[0].title == "dummy title"
    assert model.data_hashes_with_titles[0].backing_item_uuid == backing_item_uuid


class TestModelWithValidator(TestModel):
    @dto_validator(mode="before")
    def validate(cls, values):
        number: int = values.get("number_value")
        assert number > 0
        return values

    @dto_validator(mode="after")
    def validate_after(cls, instance: "TestModelWithValidator"):
        assert instance.text_value == "abc"
        instance.text_value += "-postfix"
        return instance


def test_dto_validator():
    datetime_value = datetime.now()
    data_dict = {
        "text_value": "abc",
        "number_value": 22,
        "datetime_value": format_datetime_to_long_string(datetime_value),
    }
    valid_case = TestModelWithValidator.from_dict(data_dict)
    assert valid_case.number_value == 22
    assert valid_case.text_value == f"{data_dict['text_value']}-postfix"
    assert valid_case.datetime_value == datetime_value.replace(microsecond=0).astimezone(tz=timezone.utc)

    invalid_data = data_dict
    invalid_data["number_value"] = -10

    with pytest.raises(EncordException):
        TestModelWithValidator.from_dict(invalid_data)



from encord.objects.coordinates import SkeletonCoordinate, SkeletonCoordinates, Visibility

ontology = {
    "objects": [
        {"id": "1", "name": "Triangle", "color": "#D33115", "shape": "skeleton", "featureNodeHash": "lwsBaDE4"}
    ],
    "classifications": [
        {
            "id": "1",
            "featureNodeHash": "RiMTY4XH",
            "attributes": [
                {
                    "id": "1.1",
                    "name": "Species",
                    "type": "radio",
                    "featureNodeHash": "nwC6m8CY",
                    "required": False,
                    "dynamic": False,
                    "options": [
                        {"id": "1.1.1", "label": "Cat", "value": "Cat", "featureNodeHash": "1dCoCi8n"},
                        {"id": "1.1.2", "label": "Dog", "value": "Dog", "featureNodeHash": "Z9Bb73XM"},
                    ],
                }
            ],
        }
    ],
    "skeleton_templates": [
        {
            "template": {
                "name": "Triangle",
                "shape": "skeleton",
                "width": 0.23513107100307273,
                "height": 0.25,
                "skeleton": {
                    "0": {
                        "x": 0.5148689289969273,
                        "y": 0.5,
                        "name": "point_0",
                        "color": "#000000",
                        "value": "point_0",
                        "featureHash": "A9sGmBcx",
                    },
                    "1": {
                        "x": 0.75,
                        "y": 0.5,
                        "name": "point_1",
                        "color": "#000000",
                        "value": "point_1",
                        "featureHash": "UWKgC/Dy",
                    },
                    "2": {
                        "x": 0.675,
                        "y": 0.25,
                        "name": "point_2",
                        "color": "#000000",
                        "value": "point_2",
                        "featureHash": "mBt9AAhC",
                    },
                },
                "skeletonEdges": {
                    "0": {"1": {"color": "#000000"}},
                    "1": {"2": {"color": "#000000"}},
                    "2": {"0": {"color": "#000000"}},
                },
                "feature_node_hash": "lwsBaDE4",
            }
        }
    ],
}

labels = {
    "label_hash": "1286d378-437b-4f9c-9bfa-d0bd29288faf",
    "branch_name": "main",
    "created_at": "2024-03-05 11:03:21",
    "last_edited_at": "2024-03-11 11:21:41",
    "dataset_hash": "2a8f5434-2fde-478d-9eee-14433113212e",
    "dataset_title": "20 Cats and Dogs",
    "data_title": "9970.jpg",
    "data_hash": "454da3e3-23c4-46c6-9ed4-100d9af5ba33",
    "data_type": "image",
    "is_image_sequence": False,
    "video_link": False,
    "data_units": {
        "454da3e3-23c4-46c6-9ed4-100d9af5ba33": {
            "data_hash": "454da3e3-23c4-46c6-9ed4-100d9af5ba33",
            "data_title": "9970.jpg",
            "data_type": "image/jpeg",
            "data_sequence": 0,
            "labels": {
                "objects": [
                    {
                        "featureHash": "lwsBaDE4",
                        "objectHash": "o7XU+3uz",
                        "name": "Triangle",
                        "value": "triangle",
                        "color": "#D33115",
                        "shape": "skeleton",
                        "confidence": 1,
                        "createdBy": "jim@encord.com",
                        "createdAt": "Mon, 11 Mar 2024 11:21:36 GMT",
                        "skeleton": {
                            "0": {
                                "x": 0.3838,
                                "y": 0.6729,
                                "name": "point_0",
                                "color": "#000000",
                                "value": "point_0",
                                "featureHash": "1wthOoHe",
                                "visibility": "visible",
                            },
                            "1": {
                                "x": 0.4649,
                                "y": 0.8816,
                                "name": "point_1",
                                "color": "#000000",
                                "value": "point_1",
                                "featureHash": "KGp1oToz",
                                "visibility": "occluded",
                            },
                            "2": {
                                "x": 0.2356,
                                "y": 0.9396,
                                "name": "point_2",
                                "color": "#000000",
                                "value": "point_2",
                                "featureHash": "OqR+F4dN",
                                "visibility": "invisible",
                            },
                        },
                        "manualAnnotation": True,
                    }
                ],
                "classifications": [],
            },
            "data_link": "cord-images-dev/j4uLykcoV8Xe8iogXAKxc95QPQa2/b013a65f-c553-417e-b55a-3f0b9acd5370",
            "width": 500,
            "height": 375,
        }
    },
    "object_answers": {"o7XU+3uz": {"objectHash": "o7XU+3uz", "classifications": []}},
    "classification_answers": {},
    "object_actions": {},
    "label_status": "LABELLED",
}

expected_coordinates = SkeletonCoordinates(
    values=[
        SkeletonCoordinate(
            x=0.3838,
            y=0.6729,
            name="point_0",
            color="#000000",
            feature_hash="1wthOoHe",
            value="point_0",
            visibility=Visibility.VISIBLE,
        ),
        SkeletonCoordinate(
            x=0.4649,
            y=0.8816,
            name="point_1",
            color="#000000",
            feature_hash="KGp1oToz",
            value="point_1",
            visibility=Visibility.OCCLUDED,
        ),
        SkeletonCoordinate(
            x=0.2356,
            y=0.9396,
            name="point_2",
            color="#000000",
            feature_hash="OqR+F4dN",
            value="point_2",
            visibility=Visibility.INVISIBLE,
        ),
    ],
    name="Triangle",
)



"""
Code Block Name: Automatic Interpolation
"""

# Import dependencies
from encord import EncordUserClient
from encord.objects import LabelRowV2
from encord.objects.coordinates import PointCoordinate, PolygonCoordinates

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
DATA_UNIT_TITLE = "cherries-vid-001.mp4"
DATA_UNIT_ID = "00000000-0000-0000-0000-000000000000"  # The data_hash for the data unit
LABEL_ROW_ID = "00000000-0000-0000-0000-000000000000"  # The label_hash for the data unit
LABEL_ID = "noQksGhW"  # The objectHash for the label

# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Open the project you want to work on by specifying the Project ID
project = user_client.get_project(PROJECT_ID)

# Get the label row for a specific data unit
label_row = project.get_label_row(LABEL_ROW_ID)

print(label_row)
# Prepare interpolation
key_frames = label_row["data_units"][DATA_UNIT_ID]["labels"]
objects_to_interpolate = [LABEL_ID]

# Run interpolation
interpolation_result = project.object_interpolation(key_frames, objects_to_interpolate)
print(interpolation_result)



from typing import Any, Dict
from unittest.mock import MagicMock
from uuid import uuid4

import pytest

from encord.orm.workflow import WorkflowAgentNode, WorkflowDTO
from encord.workflow import Workflow
from encord.workflow.stages.agent import AgentStage
from encord.workflow.stages.review import ReviewStage

raw_agent_workflow: Dict[str, Any] = {
    "stages": [
        {
            "stageType": "AGENT",
            "uuid": "9d6edee0-ba3e-423c-8c96-0140b07c93ed",
            "title": "Pre-labeler",
            "pathways": [
                {
                    "uuid": "ae956722-561f-4586-b6d6-145e2f64b18a",
                    "title": "Pre-labelling",
                    "destinationUuid": "1252fdb8-93cb-41a2-b341-b83ab97b5277",
                }
            ],
        },
        {"stageType": "ANNOTATION", "uuid": "1252fdb8-93cb-41a2-b341-b83ab97b5277", "title": "Annotate"},
        {"stageType": "REVIEW", "uuid": "8cac6636-2ef1-4bbe-848a-b5ffee158c34", "title": "Review"},
        {"stageType": "DONE", "uuid": "7e7598de-612c-40c4-ba08-5dfec8c3ae8f", "title": "Complete"},
    ]
}


def test_project_workflow_get_stage(workflow: Workflow) -> None:
    stage_name_to_uuid = {stage.title: stage.uuid for stage in workflow.stages}
    review_stage_uuid = stage_name_to_uuid["Review 1"]

    assert workflow.get_stage(uuid=review_stage_uuid).title == "Review 1"
    assert workflow.get_stage(uuid=str(review_stage_uuid)).title == "Review 1"
    assert workflow.get_stage(name="Review 1").uuid == review_stage_uuid
    assert isinstance(workflow.get_stage(name="Review 1"), ReviewStage)
    with pytest.raises(ValueError):
        workflow.get_stage(name="NAME NOT FOUND")
    with pytest.raises(ValueError):
        workflow.get_stage(uuid=uuid4())


def test_agent_project_with_pathway_serialisation_deserialisation() -> None:
    workflow_dto = WorkflowDTO.from_dict(raw_agent_workflow)
    assert workflow_dto.stages
    agent_stage = workflow_dto.stages[0]
    assert isinstance(agent_stage, WorkflowAgentNode)
    assert agent_stage.pathways
    workflow_obj = Workflow(MagicMock(), uuid4(), workflow_orm=workflow_dto)
    agent_stage = workflow_obj.get_stage(name="Pre-labeler")
    assert isinstance(agent_stage, AgentStage)
    assert agent_stage.pathways



labels = {
    "label_hash": "66454013-7382-4418-a9ca-41c089d8997f",
    "branch_name": "main",
    "created_at": "Thu, 09 Feb 2023 14:12:03 UTC",
    "last_edited_at": "Thu, 09 Feb 2023 14:12:03 UTC",
    "data_hash": "aaa6bc82-9f89-4545-adbb-f271bf28cf99",
    "annotation_task_status": "QUEUED",
    "is_shadow_data": False,
    "dataset_hash": "77f9c59a-89d3-4635-a20e-5da9a01d5138",
    "dataset_title": "Dataset with one single image annotated",
    "data_title": "rousseau_2.jpg",
    "data_type": "image",
    "data_units": {
        "649aba74-365b-45fa-a375-3036ee0eda78": {
            "data_hash": "649aba74-365b-45fa-a375-3036ee0eda78",
            "data_title": "rousseau_2.jpg",
            "data_type": "image/jpeg",
            "data_sequence": 0,
            "labels": {
                "objects": [
                    {
                        "name": "Nested Box",
                        "color": "#FCDC00",
                        "shape": "bounding_box",
                        "value": "nested_box",
                        "createdAt": "Tue, 17 Jan 2023 11:43:52 UTC",
                        "createdBy": "denis@cord.tech",
                        "lastEditedAt": "Tue, 17 Jan 2023 11:43:52 UTC",
                        "lastEditedBy": "denis@cord.tech",
                        "confidence": 1,
                        "objectHash": "saG3qh6C",
                        "featureHash": "MTA2MjAx",
                        "manualAnnotation": True,
                        "boundingBox": {"h": 0.1561, "w": 0.0856, "x": 0.6212, "y": 0.4114},
                    },
                    {
                        "name": "Nested Box",
                        "color": "#FCDC00",
                        "shape": "bounding_box",
                        "value": "nested_box",
                        "createdAt": "Tue, 17 Jan 2023 11:44:02 UTC",
                        "createdBy": "denis@cord.tech",
                        "lastEditedAt": "Tue, 17 Jan 2023 11:44:02 UTC",
                        "lastEditedBy": "denis@cord.tech",
                        "confidence": 1,
                        "objectHash": "2c1Uso5b",
                        "featureHash": "MTA2MjAx",
                        "manualAnnotation": True,
                        "boundingBox": {"h": 0.1336, "w": 0.1008, "x": 0.7343, "y": 0.5648},
                    },
                    {
                        "name": "Deeply Nested Polygon",
                        "color": "#DBDF00",
                        "shape": "polygon",
                        "value": "deeply_nested_polygon",
                        "polygon": {
                            "0": {"x": 0.7017, "y": 0.7857},
                            "1": {"x": 0.6895, "y": 0.7989},
                            "2": {"x": 0.6793, "y": 0.8122},
                            "3": {"x": 0.6711, "y": 0.8267},
                            "4": {"x": 0.6711, "y": 0.8439},
                            "5": {"x": 0.6742, "y": 0.8585},
                            "6": {"x": 0.6803, "y": 0.8783},
                            "7": {"x": 0.6864, "y": 0.8915},
                            "8": {"x": 0.6935, "y": 0.9021},
                            "9": {"x": 0.7017, "y": 0.9101},
                            "10": {"x": 0.7139, "y": 0.914},
                            "11": {"x": 0.7271, "y": 0.9127},
                            "12": {"x": 0.7445, "y": 0.9061},
                            "13": {"x": 0.7536, "y": 0.8981},
                            "14": {"x": 0.7689, "y": 0.8849},
                            "15": {"x": 0.7781, "y": 0.8757},
                            "16": {"x": 0.7883, "y": 0.8571},
                            "17": {"x": 0.7984, "y": 0.8333},
                            "18": {"x": 0.7995, "y": 0.8201},
                        },
                        "polygons": [
                            [
                                [
                                    0.7017,
                                    0.7857,
                                    0.6895,
                                    0.7989,
                                    0.6793,
                                    0.8122,
                                    0.6711,
                                    0.8267,
                                    0.6711,
                                    0.8439,
                                    0.6742,
                                    0.8585,
                                    0.6803,
                                    0.8783,
                                    0.6864,
                                    0.8915,
                                    0.6935,
                                    0.9021,
                                    0.7017,
                                    0.9101,
                                    0.7139,
                                    0.914,
                                    0.7271,
                                    0.9127,
                                    0.7445,
                                    0.9061,
                                    0.7536,
                                    0.8981,
                                    0.7689,
                                    0.8849,
                                    0.7781,
                                    0.8757,
                                    0.7883,
                                    0.8571,
                                    0.7984,
                                    0.8333,
                                    0.7995,
                                    0.8201,
                                ]
                            ]
                        ],
                        "createdAt": "Tue, 17 Jan 2023 11:44:38 UTC",
                        "createdBy": "denis@cord.tech",
                        "lastEditedAt": "Tue, 17 Jan 2023 11:44:38 UTC",
                        "lastEditedBy": "denis@cord.tech",
                        "confidence": 1,
                        "objectHash": "f7RpEkg5",
                        "featureHash": "MTM1MTQy",
                        "manualAnnotation": True,
                    },
                ],
                "classifications": [
                    {
                        "name": "Radio classification 1",
                        "value": "radio_classification_1",
                        "createdAt": "Tue, 17 Jan 2023 11:44:53 UTC",
                        "createdBy": "denis@cord.tech",
                        "confidence": 1,
                        "featureHash": "NzIxNTU1",
                        "lastEditedAt": "Tue, 17 Jan 2023 11:44:53 UTC",
                        "lastEditedBy": "denis@cord.tech",
                        "classificationHash": "3AqiIPrF",
                        "manualAnnotation": True,
                    },
                    {
                        "name": "Text classification",
                        "value": "text_classification",
                        "createdAt": "Tue, 17 Jan 2023 11:45:01 UTC",
                        "createdBy": "denis@cord.tech",
                        "confidence": 1,
                        "featureHash": "jPOcEsbw",
                        "lastEditedAt": "Tue, 17 Jan 2023 11:45:01 UTC",
                        "lastEditedBy": "denis@cord.tech",
                        "classificationHash": "HVwOV4hB",
                        "manualAnnotation": True,
                    },
                    {
                        "name": "Checklist classification",
                        "value": "checklist_classification",
                        "createdAt": "Tue, 17 Jan 2023 11:45:07 UTC",
                        "createdBy": "denis@cord.tech",
                        "confidence": 1,
                        "featureHash": "3DuQbFxo",
                        "lastEditedAt": "Tue, 17 Jan 2023 11:45:07 UTC",
                        "lastEditedBy": "denis@cord.tech",
                        "classificationHash": "37vDK5u0",
                        "manualAnnotation": True,
                    },
                ],
            },
            "data_link": "cord-images-dev/lFW59RQ9jcT4vHZeG14m8QWJKug1/649aba74-365b-45fa-a375-3036ee0eda78",
            "width": 1200,
            "height": 924,
        }
    },
    "object_answers": {
        "saG3qh6C": {
            "objectHash": "saG3qh6C",
            "classifications": [
                {
                    "name": "Mood",
                    "value": "mood",
                    "answers": [{"name": "Sad", "value": "sad", "featureHash": "Nzg3MDE3"}],
                    "featureHash": "ODcxMDAy",
                    "manualAnnotation": True,
                },
                {
                    "name": "First name",
                    "value": "first_name",
                    "answers": "Lion",
                    "featureHash": "OTkxMjU1",
                    "manualAnnotation": True,
                },
            ],
        },
        "2c1Uso5b": {
            "objectHash": "2c1Uso5b",
            "classifications": [
                {
                    "name": "Mood",
                    "value": "mood",
                    "answers": [
                        {"name": "Angry", "value": "angry", "featureHash": "MTE5MjQ3"},
                        {"name": "Sad", "value": "sad", "featureHash": "Nzg3MDE3"},
                    ],
                    "featureHash": "ODcxMDAy",
                    "manualAnnotation": True,
                },
                {
                    "name": "First name",
                    "value": "first_name",
                    "answers": "Aphrodite",
                    "featureHash": "OTkxMjU1",
                    "manualAnnotation": True,
                },
            ],
        },
        "f7RpEkg5": {
            "objectHash": "f7RpEkg5",
            "classifications": [
                {
                    "name": "1 2 radio 1",
                    "value": "1_2_radio_1",
                    "answers": [{"name": "1 2 1 option ", "value": "1_2_1_option_", "featureHash": "MTI4MjQy"}],
                    "featureHash": "NDYyMjQx",
                    "manualAnnotation": True,
                },
                {
                    "name": "Radio level 1 ",
                    "value": "radio_level_1_",
                    "answers": [{"name": "1 option 2", "value": "1_option_2", "featureHash": "MTcxMjAy"}],
                    "featureHash": "MTExMjI3",
                    "manualAnnotation": True,
                },
            ],
        },
    },
    "classification_answers": {
        "3AqiIPrF": {
            "classificationHash": "3AqiIPrF",
            "featureHash": "NzIxNTU1",
            "classifications": [
                {
                    "name": "cl 1 2 text",
                    "value": "cl_1_2_text",
                    "answers": "The moon is pretty",
                    "featureHash": "MTg0MjIw",
                    "manualAnnotation": True,
                },
                {
                    "name": "Radio classification 1",
                    "value": "radio_classification_1",
                    "answers": [{"name": "cl 1 option 2", "value": "cl_1_option_2", "featureHash": "MjUzMTg1"}],
                    "featureHash": "MjI5MTA5",
                    "manualAnnotation": True,
                },
            ],
        },
        "HVwOV4hB": {
            "classificationHash": "HVwOV4hB",
            "featureHash": "jPOcEsbw",
            "classifications": [
                {
                    "name": "Text classification",
                    "value": "text_classification",
                    "answers": "Rousseau",
                    "featureHash": "OxrtEM+v",
                    "manualAnnotation": True,
                }
            ],
        },
        "37vDK5u0": {
            "classificationHash": "37vDK5u0",
            "featureHash": "3DuQbFxo",
            "classifications": [
                {
                    "name": "Checklist classification",
                    "value": "checklist_classification",
                    "answers": [],
                    "featureHash": "9mwWr3OE",
                    "manualAnnotation": True,
                }
            ],
        },
    },
    "object_actions": {},
    "label_status": "LABEL_IN_PROGRESS",
}



import json
from datetime import datetime
from typing import Type, Union
from unittest.mock import MagicMock, patch

import pytest
from pydantic import BaseModel
from requests import Session

from encord.configs import SshConfig
from encord.http.v2.api_client import ApiClient
from encord.orm.base_dto import BaseDTO, RootModelDTO
from tests.fixtures import PRIVATE_KEY


class TestPayload(BaseDTO):
    payload: str


class TestPayloadV2(BaseModel):
    payload: str


class TestPayloadV3(RootModelDTO[Union[TestPayloadV2, None]]):
    pass


class TestComplexPayload(BaseDTO):
    text: str
    number: float
    time: datetime


class TestComplexPayloadV2(BaseModel):
    text: str
    number: float
    time: datetime


@pytest.fixture
def api_client():
    return ApiClient(config=SshConfig(PRIVATE_KEY))


@pytest.mark.parametrize("payload_type", [TestPayload, TestPayloadV2, TestPayloadV3])
@pytest.mark.parametrize("allow_none", [False, True])
@patch.object(Session, "send")
def test_constructed_url_is_correct(
    send: MagicMock, api_client: ApiClient, payload_type: Type[TestPayload], allow_none: bool
) -> None:
    mock_response = MagicMock()
    mock_response.status_code = 200
    mock_response.json.return_value = None if allow_none else {"payload": "hello world"}
    mock_response.content = "null" if allow_none else json.dumps({"payload": "hello world"})
    send.return_value = mock_response

    res = [
        api_client.get("/", params=None, result_type=payload_type, allow_none=allow_none),
        api_client.get("/test", params=None, result_type=payload_type, allow_none=allow_none),
        api_client.get("/test/", params=None, result_type=payload_type, allow_none=allow_none),
        api_client.get("test/", params=None, result_type=payload_type, allow_none=allow_none),
    ]
    if allow_none:
        for r in res:
            assert r is None
    else:
        for r in res:
            assert r is not None
            if payload_type == TestPayloadV3:
                assert r.root.payload == "hello world"
            else:
                assert r.payload == "hello world"

    assert send.call_args_list[0].args[0].url == "https://api.encord.com/v2/public"
    assert send.call_args_list[1].args[0].url == "https://api.encord.com/v2/public/test"
    assert send.call_args_list[2].args[0].url == "https://api.encord.com/v2/public/test"
    assert send.call_args_list[3].args[0].url == "https://api.encord.com/v2/public/test"


@pytest.mark.parametrize("payload_type", [TestPayload, TestPayloadV2, TestPayloadV3])
@pytest.mark.parametrize("allow_none", [False, True])
@patch.object(Session, "send")
def test_payload_url_serialisation(
    send: MagicMock, api_client: ApiClient, payload_type: Type[TestPayload], allow_none: bool
) -> None:
    mock_response = MagicMock()
    mock_response.status_code = 200
    mock_response.json.return_value = None if allow_none else {"payload": "hello world"}
    mock_response.content = "null" if allow_none else json.dumps({"payload": "hello world"})
    send.return_value = mock_response

    expected_time = datetime.fromisoformat("2024-02-01T01:02:03+01:00")
    payload = TestComplexPayload(text="test", number=0.01, time=expected_time)

    res = api_client.get("/", params=payload, result_type=payload_type, allow_none=allow_none)
    if allow_none:
        assert res is None
    else:
        assert res is not None
        if payload_type == TestPayloadV3:
            assert res.root.payload == "hello world"
        else:
            assert res.payload == "hello world"

    assert (
        send.call_args_list[0].args[0].url
        == "https://api.encord.com/v2/public?text=test&number=0.01&time=2024-02-01T01%3A02%3A03%2B01%3A00"
    )


@pytest.mark.parametrize("payload_type", [TestPayload, TestPayloadV2, TestPayloadV3, None])
@pytest.mark.parametrize("param_type", [TestComplexPayload, TestComplexPayloadV2])
@patch.object(Session, "send")
def test_payload_body_serialisation(
    send: MagicMock,
    api_client: ApiClient,
    payload_type: Type[TestPayload],
    param_type: Type[TestComplexPayload],
) -> None:
    mock_response = MagicMock()
    mock_response.status_code = 200
    mock_response.json.return_value = None if payload_type is None else {"payload": "hello world"}
    mock_response.content = "null" if payload_type is None else json.dumps({"payload": "hello world"})
    send.return_value = mock_response

    expected_time = datetime.fromisoformat("2024-02-01T01:02:03+01:00")
    payload = param_type(text="test", number=0.01, time=expected_time)

    r = api_client.post("/", params=None, payload=payload, result_type=payload_type)
    if payload_type is None:
        assert r is None
    else:
        assert r is not None
        if payload_type == TestPayloadV3:
            assert r.root.payload == "hello world"
        else:
            assert r.payload == "hello world"

    assert send.call_args_list[0].args[0].url == "https://api.encord.com/v2/public"
    assert (
        send.call_args_list[0].args[0].body == b'{"text": "test", "number": 0.01, "time": "2024-02-01T01:02:03+01:00"}'
    )



"""
Code Block Name: Bitmasks PDFs
"""

# Import dependencies
from encord import EncordUserClient, Project
from encord.objects import ChecklistAttribute, Object, ObjectInstance, Option, RadioAttribute, TextAttribute
from encord.objects.coordinates import BoundingBoxCoordinates

SSH_PATH = "/Users/chris-encord/staging-sdk-ssh-key-private-key.txt"
# SSH_PATH = get_ssh_key() # replace it with ssh key
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
BUNDLE_SIZE = 100

# Create user client
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Get project
project: Project = user_client.get_project(PROJECT_ID)
ontology_structure = project.ontology_structure

# Get ontology object
box_ontology_object: Object = ontology_structure.get_child_by_title(title="PDF BB", type_=Object)
assert box_ontology_object is not None, "Bounding box object 'PDF BB' not found in ontology."

# Define radio attribute for correction types
correction_radio_attribute = ontology_structure.get_child_by_title(type_=RadioAttribute, title="Corrections BB")
assert correction_radio_attribute is not None, "Radio attribute 'Corrections BB' not found in ontology."

english_correction_option = correction_radio_attribute.get_child_by_title(type_=Option, title="English corrections")
chinese_correction_option = correction_radio_attribute.get_child_by_title(type_=Option, title="繁體中文修正")
assert all([english_correction_option, chinese_correction_option]), (
    "One or more correction type options missing in radio attribute."
)

# Define checklist attributes
english_checklist_attribute = ontology_structure.get_child_by_title(type_=ChecklistAttribute, title="English")
assert english_checklist_attribute is not None, "Checklist attribute 'English' not found in ontology."

en_ca_option = english_checklist_attribute.get_child_by_title(type_=Option, title="en-ca")
en_gb_option = english_checklist_attribute.get_child_by_title(type_=Option, title="en-gb")
en_us_option = english_checklist_attribute.get_child_by_title(type_=Option, title="en-us")
assert all([en_ca_option, en_gb_option, en_us_option]), "One or more English checklist options are missing."

chinese_checklist_attribute = ontology_structure.get_child_by_title(type_=ChecklistAttribute, title="繁體中文")
assert chinese_checklist_attribute is not None, "Checklist attribute '繁體中文' not found in ontology."

zh_tw_option = chinese_checklist_attribute.get_child_by_title(type_=Option, title="zh-tw")
zh_hk_option = chinese_checklist_attribute.get_child_by_title(type_=Option, title="zh-hk")
assert all([zh_tw_option, zh_hk_option]), "One or more Chinese checklist options are missing."

# Define text attributes
english_correction_text_attribute = ontology_structure.get_child_by_title(type_=TextAttribute, title="Correction text")
assert english_correction_text_attribute is not None, "Text attribute 'Correction text' not found in ontology."

chinese_correction_text_attribute = ontology_structure.get_child_by_title(type_=TextAttribute, title="更正")
assert chinese_correction_text_attribute is not None, "Text attribute '更正' not found in ontology."

# Example label data (you can adjust this)
pdf_labels = {
    "the-iliad.pdf": {
        # Specify the page number in the PDF. In the example below, page number 103 is labeled
        103: {
            "label_ref": "pdf_label_001",
            "coordinates": BoundingBoxCoordinates(height=0.4, width=0.4, top_left_x=0.1, top_left_y=0.1),
            "correction_type": "English corrections",
            "checklist_options": "en-ca, en-gb",
            "text_correction": "Fixed typo in English text.",
        }
    },
    "dracula.pdf": {
        # Specify the page number in the PDF. In the example below, page number 17 is labeled
        17: {
            "label_ref": "pdf_label_002",
            "coordinates": BoundingBoxCoordinates(height=0.3, width=0.5, top_left_x=0.2, top_left_y=0.2),
            "correction_type": "繁體中文修正",
            "checklist_options": "zh-tw",
            "text_correction": "修正了中文繁體的標點符號。",
        }
    },
}

# === Step 1: Initialize all label rows using a bundle ===

label_row_map = {}

with project.create_bundle(bundle_size=BUNDLE_SIZE) as bundle:
    for data_unit in pdf_labels.keys():
        label_rows = project.list_label_rows_v2(data_title_eq=data_unit)
        assert isinstance(label_rows, list), f"[ASSERT] Expected a list of label rows for {data_unit}"
        if not label_rows:
            print(f"[SKIP] No label row found for {data_unit}")
            continue

        label_row = label_rows[0]
        try:
            label_row.initialise_labels(bundle=bundle)
            label_row_map[data_unit] = label_row
            print(f"[INIT] Initialized label row for: {data_unit}")
        except Exception as e:
            raise AssertionError(f"[ASSERT] Failed to initialize label row for {data_unit}: {e}")

# === Step 2: Process all frames/annotations and prepare label rows to save ===

for data_unit, frame_coordinates in pdf_labels.items():
    label_row = label_row_map.get(data_unit)
    assert label_row is not None, f"[ASSERT] Missing initialized label row for {data_unit}"

    object_instances_by_label_ref = {}

    for frame_number, items in frame_coordinates.items():
        if not isinstance(items, list):  # Handle single vs list
            items = [items]

        for item in items:
            label_ref = item["label_ref"]
            coord = item["coordinates"]
            correction_type = item["correction_type"]
            checklist_options_str = item.get("checklist_options", "")
            text_correction = item.get("text_correction", "")

            if label_ref not in object_instances_by_label_ref:
                box_object_instance: ObjectInstance = box_ontology_object.create_instance()
                assert box_object_instance is not None, f"[ASSERT] Failed to create ObjectInstance for {label_ref}"
                object_instances_by_label_ref[label_ref] = box_object_instance

                if correction_type == "English corrections":
                    assert english_correction_option is not None, "[ASSERT] english_correction_option not defined"
                    box_object_instance.set_answer(
                        attribute=correction_radio_attribute, answer=english_correction_option
                    )

                    checklist_answers = []
                    for option in [opt.strip() for opt in checklist_options_str.split(",")]:
                        if option == "en-ca":
                            checklist_answers.append(en_ca_option)
                        elif option == "en-gb":
                            checklist_answers.append(en_gb_option)
                        elif option == "en-us":
                            checklist_answers.append(en_us_option)
                        else:
                            raise AssertionError(f"[ASSERT] Unknown English checklist option: {option}")

                    if checklist_answers:
                        assert english_checklist_attribute is not None, (
                            "[ASSERT] english_checklist_attribute not defined"
                        )
                        box_object_instance.set_answer(
                            attribute=english_checklist_attribute,
                            answer=checklist_answers,
                            overwrite=True,
                        )

                    if text_correction:
                        assert english_correction_text_attribute is not None, (
                            "[ASSERT] english_correction_text_attribute not defined"
                        )
                        box_object_instance.set_answer(
                            attribute=english_correction_text_attribute, answer=text_correction
                        )

                elif correction_type == "繁體中文修正":
                    assert chinese_correction_option is not None, "[ASSERT] chinese_correction_option not defined"
                    box_object_instance.set_answer(
                        attribute=correction_radio_attribute, answer=chinese_correction_option
                    )

                    checklist_answers = []
                    for option in [opt.strip() for opt in checklist_options_str.split(",")]:
                        if option == "zh-tw":
                            checklist_answers.append(zh_tw_option)
                        elif option == "zh-hk":
                            checklist_answers.append(zh_hk_option)
                        else:
                            raise AssertionError(f"[ASSERT] Unknown Chinese checklist option: {option}")

                    if checklist_answers:
                        assert chinese_checklist_attribute is not None, (
                            "[ASSERT] chinese_checklist_attribute not defined"
                        )
                        box_object_instance.set_answer(
                            attribute=chinese_checklist_attribute,
                            answer=checklist_answers,
                            overwrite=True,
                        )

                    if text_correction:
                        assert chinese_correction_text_attribute is not None, (
                            "[ASSERT] chinese_correction_text_attribute not defined"
                        )
                        box_object_instance.set_answer(
                            attribute=chinese_correction_text_attribute, answer=text_correction
                        )

            else:
                # Reuse existing instance
                box_object_instance = object_instances_by_label_ref[label_ref]

            box_object_instance.set_for_frames(coordinates=coord, frames=frame_number)

    for box_object_instance in object_instances_by_label_ref.values():
        assert box_object_instance.get_annotation_frames(), "[ASSERT] Object instance has no frames assigned"
        label_row.add_object_instance(box_object_instance)

    try:
        label_row.save()
        print(f"[SAVE] Label row saved for {data_unit}")
    except Exception as e:
        raise AssertionError(f"[ASSERT] Failed to save label row for {data_unit}: {e}")

print("\n[COMPLETE] Labels with English and Mandarin corrections have been added for all data units.")



PLAIN_TEXT_LABELS = {
    "label_hash": "0aea5ac7-cbc0-4451-a242-e22445d2c9fa",
    "branch_name": "main",
    "created_at": "Thu, 09 Feb 2023 14:12:03 UTC",
    "last_edited_at": "Thu, 09 Feb 2023 14:12:03 UTC",
    "data_hash": "aaa6bc82-9f89-4545-adbb-f271bf28cf99",
    "annotation_task_status": "QUEUED",
    "is_shadow_data": False,
    "dataset_hash": "b02ba3d9-883b-4c5e-ba09-751072ccfc57",
    "dataset_title": "Text Dataset",
    "data_title": "text.txt",
    "data_type": "plain_text",
    "data_units": {
        "cd53f484-c9ab-4fd1-9c14-5b34d4e42ba2": {
            "data_hash": "cd53f484-c9ab-4fd1-9c14-5b34d4e42ba2",
            "data_title": "text.txt",
            "data_type": "text/plain",
            "data_sequence": 0,
            "data_link": "text-link",
            "labels": {},
        }
    },
    "object_answers": {
        "textObjectHash": {
            "objectHash": "textObjectHash",
            "featureHash": "textObjectFeatureNodeHash",
            "classifications": [],
            "range": [[0, 5]],
            "createdBy": "user1Hash",
            "createdAt": "Tue, 05 Nov 2024 09:41:37 UTC",
            "lastEditedBy": "user1Hash",
            "lastEditedAt": "Tue, 05 Nov 2024 09:41:37 UTC",
            "manualAnnotation": True,
            "name": "text object",
            "value": "text_object",
            "color": "#A4DD00",
            "shape": "text",
        },
    },
    "classification_answers": {
        "textClassificationHash": {
            "classificationHash": "textClassificationHash",
            "featureHash": "jPOcEsbw",
            "classifications": [
                {
                    "name": "Text classification",
                    "value": "text_classification",
                    "answers": "Text Answer",
                    "featureHash": "OxrtEM+v",
                    "manualAnnotation": True,
                },
            ],
            "range": [],
            "createdBy": "user1Hash",
            "createdAt": "Tue, 05 Nov 2024 09:41:37 UTC",
            "lastEditedBy": "user1Hash",
            "lastEditedAt": "Tue, 05 Nov 2024 09:41:37 UTC",
            "manualAnnotation": True,
        },
        "radioClassificationHash": {
            "classificationHash": "radioClassificationHash",
            "featureHash": "NzIxNTU1",
            "classifications": [
                {
                    "name": "Radio classification 1",
                    "value": "radio_classification_1",
                    "answers": [
                        {
                            "name": "cl 1 option 1",
                            "value": "cl_1_option_1",
                            "featureHash": "MTcwMjM5",
                        }
                    ],
                    "featureHash": "MjI5MTA5",
                    "manualAnnotation": True,
                },
                {
                    "name": "cl 1 2 text",
                    "value": "cl_1_2_text",
                    "answers": "Nested Text Answer",
                    "featureHash": "MTg0MjIw",
                    "manualAnnotation": True,
                },
            ],
            "range": [],
            "createdBy": "user1Hash",
            "createdAt": "Tue, 05 Nov 2024 09:41:37 UTC",
            "lastEditedBy": "user1Hash",
            "lastEditedAt": "Tue, 05 Nov 2024 09:41:37 UTC",
            "manualAnnotation": True,
        },
        "checklistClassificationHash": {
            "classificationHash": "checklistClassificationHash",
            "featureHash": "3DuQbFxo",
            "classifications": [
                {
                    "name": "Checklist classification",
                    "value": "checklist_classification",
                    "answers": [
                        {
                            "name": "Checklist classification answer 1",
                            "value": "checklist_classification_answer_1",
                            "featureHash": "fvLjF0qZ",
                        },
                        {
                            "name": "Checklist classification answer 2",
                            "value": "checklist_classification_answer_2",
                            "featureHash": "a4r7nK9i",
                        },
                    ],
                    "featureHash": "9mwWr3OE",
                    "manualAnnotation": True,
                },
            ],
            "range": [],
            "createdBy": "user1Hash",
            "createdAt": "Tue, 05 Nov 2024 09:41:37 UTC",
            "lastEditedBy": "user1Hash",
            "lastEditedAt": "Tue, 05 Nov 2024 09:41:37 UTC",
            "manualAnnotation": True,
        },
    },
    "object_actions": {},
    "label_status": "LABEL_IN_PROGRESS",
}

EMPTY_PLAIN_TEXT_LABELS = {
    "label_hash": "0aea5ac7-cbc0-4451-a242-e22445d2c9fa",
    "branch_name": "main",
    "created_at": "2023-02-09 14:12:03",
    "last_edited_at": "2023-02-09 14:12:03",
    "data_hash": "aaa6bc82-9f89-4545-adbb-f271bf28cf99",
    "annotation_task_status": "QUEUED",
    "is_shadow_data": False,
    "dataset_hash": "b02ba3d9-883b-4c5e-ba09-751072ccfc57",
    "dataset_title": "Text Dataset",
    "data_title": "text.txt",
    "data_type": "plain_text",
    "data_units": {
        "cd53f484-c9ab-4fd1-9c14-5b34d4e42ba2": {
            "data_hash": "cd53f484-c9ab-4fd1-9c14-5b34d4e42ba2",
            "data_title": "text.txt",
            "data_type": "text/plain",
            "data_sequence": 0,
            "data_link": "text-link",
            "labels": {},
        }
    },
    "object_answers": {},
    "classification_answers": {},
    "object_actions": {},
    "label_status": "LABEL_IN_PROGRESS",
}



"""
Code Block Name: Verify where Project files are stored
"""

import json

from encord import EncordUserClient

# User input
SSH_PATH = "/Users/chris-encord/ssh-private-key.txt"
PROJECT_ID = "00000000-0000-0000-0000-000000000000"
OUTPUT_FILE_PATH = "/Users/chris-encord/file-locations-output.json"  # Specify the path where you want to save the JSON

# Create user client using SSH key
user_client: EncordUserClient = EncordUserClient.create_with_ssh_private_key(
    ssh_private_key_path=SSH_PATH,
    # For US platform users use "https://api.us.encord.com"
    domain="https://api.encord.com",
)

# Specify Project. Replace <project_id> with the ID of the Project you want to export labels for.
project = user_client.get_project(PROJECT_ID)

# Collect data to be saved
output_data = []

# Retrieve and store the label rows
for log_line in project.list_label_rows_v2():
    data_list = project.get_data(log_line.data_hash, get_signed_url=True)
    output_data.append(data_list)

# Save output data to JSON file
with open(OUTPUT_FILE_PATH, "w") as json_file:
    json.dump(output_data, json_file, indent=4)

print(f"Data successfully saved to {OUTPUT_FILE_PATH}")
